@@ -16,14 +16,14 @@
  
 Copyright 2001 Peter Beerli and Joseph Felsenstein
 
-$Id: data.c,v 1.35 2001/06/03 20:29:02 beerli Exp $
+$Id: data.c,v 1.37 2001/09/07 23:56:13 beerli Exp $
 
 -------------------------------------------------------*/
 #include <string.h>
 
 #include "migration.h"
 
-extern long number_genomes(char type);
+extern long number_genomes (char type);
 
 #ifdef DMALLOC_FUNC_CHECK
 #include <dmalloc.h>
@@ -31,8 +31,9 @@ extern long number_genomes(char type);
 /* prototypes ----------------------------------------- */
 void create_data (data_fmt ** data);
 void get_data (FILE * infile, data_fmt * data, option_fmt * options);
-void print_data (world_fmt * world, option_fmt * options, data_fmt *data);
-void print_data_summary (FILE * file, world_fmt * world,  option_fmt * options,data_fmt * data);
+void print_data (world_fmt * world, option_fmt * options, data_fmt * data);
+void print_data_summary (FILE * file, world_fmt * world, option_fmt * options,
+			 data_fmt * data);
 void free_datapart (data_fmt * data, option_fmt * options, long locus);
 /*private functions */
 void init_data_structure1 (data_fmt ** data, option_fmt * options);
@@ -41,26 +42,37 @@ void read_sites (data_fmt * data);
 void init_data_structure2 (data_fmt ** data, option_fmt * options, long pop);
 void init_data_structure3 (data_fmt * data);
 void read_popheader (FILE * infile, data_fmt * data, long pop);
-void read_indname (FILE * file, data_fmt * data, long pop, long ind, long nmlength);
-void read_popdata (FILE * file, data_fmt * data, long pop, option_fmt * options);
+void read_indname (FILE * file, data_fmt * data, long pop, long ind,
+		   long nmlength);
+void read_popdata (FILE * file, data_fmt * data, long pop,
+		   option_fmt * options);
 void read_microalleles (FILE * infile, data_fmt * data, long pop, long ind);
 void read_alleles (FILE * infile, data_fmt * data, long pop, long ind);
-long read_ind_seq (FILE * infile, data_fmt * data, option_fmt * options, long locus, long pop, long ind, long baseread);
-void read_distance_fromfile (FILE * dfile, long tips, long nmlength, double **m);
-void finish_read_seq (FILE * infile, data_fmt * data, option_fmt * options, long pop, long baseread);
-void print_alleledata (world_fmt * world, data_fmt * data, option_fmt * options);
+long read_ind_seq (FILE * infile, data_fmt * data, option_fmt * options,
+		   long locus, long pop, long ind, long baseread);
+void read_distance_fromfile (FILE * dfile, long tips, long nmlength,
+			     double **m);
+void finish_read_seq (FILE * infile, data_fmt * data, option_fmt * options,
+		      long pop, long baseread);
+void print_alleledata (world_fmt * world, data_fmt * data,
+		       option_fmt * options);
 void print_seqdata (world_fmt * world, option_fmt * options, data_fmt * data);
 
-void print_header (FILE * outfile, long pop, world_fmt * world, option_fmt * options, data_fmt * data);
+void print_header (FILE * outfile, long pop, world_fmt * world,
+		   option_fmt * options, data_fmt * data);
 void create_alleles (data_fmt * data);
 void addAllele (data_fmt * data, char s[], long locus, long *z);
 void set_numind (data_fmt * data);
-void print_seq_pop (long locus, long pop, world_fmt * world, option_fmt * options, data_fmt * data);
-void print_seq_ind (long locus, long pop, long ind, world_fmt * world, option_fmt * options, data_fmt * data);
-void print_locus_head (long locus, world_fmt * world, option_fmt * options, data_fmt * data);
+void print_seq_pop (long locus, long pop, world_fmt * world,
+		    option_fmt * options, data_fmt * data);
+void print_seq_ind (long locus, long pop, long ind, world_fmt * world,
+		    option_fmt * options, data_fmt * data);
+void print_locus_head (long locus, world_fmt * world, option_fmt * options,
+		       data_fmt * data);
 void find_delimiter (char *title, char *dlm);
 void read_geofile (data_fmt * data, option_fmt * options, long numpop);
-void read_uep_fromfile (FILE * uepfile, long tips, long nmlength, long **uep, long *uepsites, long datatype);
+void read_uep_fromfile (FILE * uepfile, long tips, long nmlength, long **uep,
+			long *uepsites, long datatype);
 void read_uepfile (data_fmt * data, option_fmt * options, long numpop);
 
 
@@ -156,17 +168,23 @@ init_data_structure1 (data_fmt ** data, option_fmt * options)
   long pop;
   if ((*data)->yy == NULL)
     {
-      (*data)->yy = (char *****) malloc (sizeof (char ****) * (*data)->numpop);
+      (*data)->yy =
+	(char *****) malloc (sizeof (char ****) * (*data)->numpop);
       (*data)->seq = (seqmodel_fmt *) calloc (1, sizeof (seqmodel_fmt));
-      (*data)->popnames = (char **) malloc (sizeof (char *) * (*data)->numpop);
-      (*data)->indnames = (char ***) malloc (sizeof (char **) * (*data)->numpop);
+      (*data)->popnames =
+	(char **) malloc (sizeof (char *) * (*data)->numpop);
+      (*data)->indnames =
+	(char ***) malloc (sizeof (char **) * (*data)->numpop);
       (*data)->numind = (long **) malloc (sizeof (long *) * (*data)->numpop);
       for (pop = 0; pop < (*data)->numpop; pop++)
 	{
-	  (*data)->popnames[pop] = (char *) calloc (1, sizeof (char) * LINESIZE);
-	  (*data)->numind[pop] = (long *) malloc (sizeof (long) * (*data)->loci);
+	  (*data)->popnames[pop] =
+	    (char *) calloc (1, sizeof (char) * LINESIZE);
+	  (*data)->numind[pop] =
+	    (long *) malloc (sizeof (long) * (*data)->loci);
 	}
-      (*data)->seq->sites = (long *) calloc (1, sizeof (long) * (*data)->loci);
+      (*data)->seq->sites =
+	(long *) calloc (1, sizeof (long) * (*data)->loci);
     }
   else
     {
@@ -186,20 +204,28 @@ init_data_structure2 (data_fmt ** data, option_fmt * options, long pop)
   (*data)->indnames[pop] = (char **) calloc (1, sizeof (char *) * indalloc);
   for (ind = 0; ind < indalloc; ind++)
     {
-      (*data)->indnames[pop][ind] = (char *) calloc (1, sizeof (char) * (1 + options->nmlength));
-      (*data)->yy[pop][ind] = (char ***) malloc (sizeof (char **) * (*data)->loci);
+      (*data)->indnames[pop][ind] =
+	(char *) calloc (1, sizeof (char) * (1 + options->nmlength));
+      (*data)->yy[pop][ind] =
+	(char ***) malloc (sizeof (char **) * (*data)->loci);
       for (locus = 0; locus < (*data)->loci; locus++)
 	{
-	  if (!strchr(SEQUENCETYPES,options->datatype))
+	  if (!strchr (SEQUENCETYPES, options->datatype))
 	    {
-	      (*data)->yy[pop][ind][locus] = (char **) calloc (1, sizeof (char *) * 2);
-	      (*data)->yy[pop][ind][locus][0] = (char *) calloc (1, sizeof (char) * options->allelenmlength);
-	      (*data)->yy[pop][ind][locus][1] = (char *) calloc (1, sizeof (char) * options->allelenmlength);
+	      (*data)->yy[pop][ind][locus] =
+		(char **) calloc (1, sizeof (char *) * 2);
+	      (*data)->yy[pop][ind][locus][0] =
+		(char *) calloc (1, sizeof (char) * options->allelenmlength);
+	      (*data)->yy[pop][ind][locus][1] =
+		(char *) calloc (1, sizeof (char) * options->allelenmlength);
 	    }
 	  else
 	    {
-	      (*data)->yy[pop][ind][locus] = (char **) calloc (1, sizeof (char *));
-	      (*data)->yy[pop][ind][locus][0] = (char *) calloc (1, sizeof (char) * (*data)->seq->sites[locus]);
+	      (*data)->yy[pop][ind][locus] =
+		(char **) calloc (1, sizeof (char *));
+	      (*data)->yy[pop][ind][locus][0] =
+		(char *) calloc (1,
+				 sizeof (char) * (*data)->seq->sites[locus]);
 	    }
 	}
     }
@@ -210,12 +236,12 @@ void
 free_datapart (data_fmt * data, option_fmt * options, long locus)
 {
   long ind, pop;
-  long genomes = number_genomes(options->datatype);
+  long genomes = number_genomes (options->datatype);
   for (pop = 0; pop < data->numpop; pop++)
     {
       for (ind = 0; ind < data->numind[pop][locus] / genomes; ind++)
 	{
-	  if (strchr(SEQUENCETYPES,options->datatype))
+	  if (strchr (SEQUENCETYPES, options->datatype))
 	    {
 	      free (data->yy[pop][ind][locus][0]);
 	      free (data->yy[pop][ind][locus][1]);
@@ -235,16 +261,19 @@ void
 init_data_structure3 (data_fmt * data)
 {
   long locus, pop, maxi;
-  data->allele = (allele_fmt **) calloc (1, sizeof (allele_fmt *) * data->loci);
+  data->allele =
+    (allele_fmt **) calloc (1, sizeof (allele_fmt *) * data->loci);
   for (locus = 0; locus < data->loci; locus++)
     {
       maxi = 0;
       for (pop = 0; pop < data->numpop; pop++)
 	maxi += data->numind[pop][FLOC] * 2;
-      data->allele[locus] = (allele_fmt *) calloc (1, sizeof (allele_fmt) * maxi);
+      data->allele[locus] =
+	(allele_fmt *) calloc (1, sizeof (allele_fmt) * maxi);
     }
   data->maxalleles = (long *) calloc (1, sizeof (long) * data->loci);
-  data->skiploci = (boolean *) calloc (1, sizeof (boolean) * (data->loci + 1));
+  data->skiploci =
+    (boolean *) calloc (1, sizeof (boolean) * (data->loci + 1));
 }
 
 
@@ -258,18 +287,21 @@ read_header (FILE * infile, data_fmt * data, option_fmt * options)
   switch (lowercase (input[0]))
     {
     case 'a':
-      sscanf (input, "%1s%ld%ld%[^\n]", &options->datatype, &(data->numpop), &(data->loci), options->title);
+      sscanf (input, "%1s%ld%ld%[^\n]", &options->datatype, &(data->numpop),
+	      &(data->loci), options->title);
       find_delimiter (options->title, &data->dlm);
       break;
     case 'b':
     case 'm':
-      sscanf (input, "%1s%ld%ld%1s%[^\n]", &options->datatype, &(data->numpop), &(data->loci), &data->dlm, options->title);
+      sscanf (input, "%1s%ld%ld%1s%[^\n]", &options->datatype,
+	      &(data->numpop), &(data->loci), &data->dlm, options->title);
       break;
     case 's':
     case 'n':
     case 'u':
     case 'f':
-      sscanf (input, "%1s%ld%ld%[^\n]", &options->datatype, &(data->numpop), &(data->loci), options->title);
+      sscanf (input, "%1s%ld%ld%[^\n]", &options->datatype, &(data->numpop),
+	      &(data->loci), options->title);
       break;
     case 'g':			/* fall through if a menu change forces to analyze data
 				   instead of using the already sampled genealogies */
@@ -281,18 +313,21 @@ read_header (FILE * infile, data_fmt * data, option_fmt * options)
       switch (options->datatype)
 	{
 	case 'a':
-	  sscanf (input, "%ld%ld%[^\n]", &(data->numpop), &(data->loci), options->title);
+	  sscanf (input, "%ld%ld%[^\n]", &(data->numpop), &(data->loci),
+		  options->title);
 	  find_delimiter (options->title, &data->dlm);
 	  break;
 	case 'b':
 	case 'm':
-	  sscanf (input, "%ld%ld%1s%[^\n]", &(data->numpop), &(data->loci), &(data->dlm), options->title);
+	  sscanf (input, "%ld%ld%1s%[^\n]", &(data->numpop), &(data->loci),
+		  &(data->dlm), options->title);
 	  break;
 	case 's':
 	case 'n':
 	case 'u':
 	case 'f':
-	  sscanf (input, "%ld%ld%[^\n]", &(data->numpop), &(data->loci), options->title);
+	  sscanf (input, "%ld%ld%[^\n]", &(data->numpop), &(data->loci),
+		  options->title);
 	  break;
 	default:
 	  usererror ("Datatype is wrong, please use a valid data type!");
@@ -365,7 +400,8 @@ read_popheader (FILE * infile, data_fmt * data, long pop)
   FGETS (input, sizeof (input), infile);
   if ((p = (char *) strpbrk (input, CRLF)) != NULL)
     *p = '\0';
-  sscanf (input, "%ld%[^\n]", &(data->numind[pop][FLOC]), data->popnames[pop]);
+  sscanf (input, "%ld%[^\n]", &(data->numind[pop][FLOC]),
+	  data->popnames[pop]);
   translate (data->popnames[pop], ' ', '_');
   translate (data->popnames[pop], '\t', '_');
   if (data->numind[pop][FLOC] == 0)
@@ -406,11 +442,12 @@ read_popdata (FILE * infile, data_fmt * data, long pop, option_fmt * options)
 	  baseread = read_ind_seq (infile, data, options, locus, pop, ind, 0);
 	  break;
 	default:
-	  usererror ("Wrong datatype, only the types a, m, s, n\n       (electrophoretic alleles, \n       microsatellite data,\n       sequence data,\n       SNP polymorphism)\n        are allowed.\n");
+	  usererror
+	    ("Wrong datatype, only the types a, m, s, n\n       (electrophoretic alleles, \n       microsatellite data,\n       sequence data,\n       SNP polymorphism)\n        are allowed.\n");
 	  break;
 	}
     }
-  if (!strchr(SEQUENCETYPES,options->datatype))
+  if (!strchr (SEQUENCETYPES, options->datatype))
     return;
   else
     {
@@ -517,14 +554,16 @@ read_alleles (FILE * infile, data_fmt * data, long pop, long ind)
 }
 
 long
-read_ind_seq (FILE * infile, data_fmt * data, option_fmt * options, long locus, long pop, long ind, long baseread)
+read_ind_seq (FILE * infile, data_fmt * data, option_fmt * options,
+	      long locus, long pop, long ind, long baseread)
 {
   long j;
   char charstate;
   j = (options->interleaved) ? baseread : 0;
   charstate = getc (infile);
   ungetc ((int) charstate, infile);
-  while (j < data->seq->sites[locus] && !(options->interleaved && charstate == '\n'))
+  while (j < data->seq->sites[locus]
+	 && !(options->interleaved && charstate == '\n'))
     {
       charstate = getc (infile);
       if (charstate == '\n')
@@ -534,23 +573,28 @@ read_ind_seq (FILE * infile, data_fmt * data, option_fmt * options, long locus,
 	  else
 	    charstate = ' ';
 	}
-      if (charstate=='\r' || charstate == ' ' || (charstate >= '0' && charstate <= '9') || charstate == '\\')
+      if (charstate == '\r' || charstate == ' '
+	  || (charstate >= '0' && charstate <= '9') || charstate == '\\')
 	continue;
       charstate = uppercase (charstate);
       //      printf("%c",charstate);
       if ((strchr ("ABCDGHKMNRSTUVWXY?O-", (int) charstate)) == NULL)
 	{
-	  printf ("ERROR: BAD BASE: %c AT POSITION %5ld OF INDIVIDUUM %3li in POPULATION %ld\n", charstate, j, ind, pop);
-	  printf ("Last complete sequences was in population %li, individual %li and locus %li:\n%s", pop + 1, ind - 1, locus, data->indnames[pop][ind - 1]);
+	  printf
+	    ("ERROR: BAD BASE: %c AT POSITION %5ld OF INDIVIDUUM %3li in POPULATION %ld\n",
+	     charstate, j, ind, pop);
+	  printf
+	    ("Last complete sequences was in population %li, individual %li and locus %li:\n%s",
+	     pop + 1, ind - 1, locus, data->indnames[pop][ind - 1]);
 	  for (j = 0; j < data->seq->sites[locus]; j++)
 	    printf ("%c", data->yy[pop][ind - 1][locus][0][j]);
 	  exit (EXIT_FAILURE);
 	}
       data->yy[pop][ind][locus][0][j++] = charstate;
     }
-  charstate = getc (infile);	/* swallow the \n or \r*/
+  charstate = getc (infile);	/* swallow the \n or \r */
 #ifndef MAC
-  if(charstate=='\r')
+  if (charstate == '\r')
     charstate = getc (infile);	/* swallow the \n */
 #endif
   return j;
@@ -586,7 +630,8 @@ read_distance_fromfile (FILE * dfile, long tips, long nmlength, double **m)
 // uep function 
 
 void
-read_uep_fromfile (FILE * uepfile, long tips, long nmlength, long **uep, long *uepsites, long datatype)
+read_uep_fromfile (FILE * uepfile, long tips, long nmlength, long **uep,
+		   long *uepsites, long datatype)
 {
   char input[LINESIZE];
   long i, j;
@@ -600,12 +645,12 @@ read_uep_fromfile (FILE * uepfile, long tips, long nmlength, long **uep, long *u
       sscanf (input, "%li%li", &thistips, uepsites);
       if (thistips != tips)
 	error ("UEP datafile and infile are inconsistent!");
-      if(strchr(SEQUENCETYPES,datatype))
+      if (strchr (SEQUENCETYPES, datatype))
 	{
 	  for (i = 0; i < tips; i++)
 	    {
 	      uep[i] = (long *) calloc (*uepsites, sizeof (long));
-	      FGETS (input, nmlength + 1, uepfile);  //reads each line
+	      FGETS (input, nmlength + 1, uepfile);	//reads each line
 	      for (j = 0; j < *uepsites; ++j)
 		fscanf (uepfile, "%li", &uep[i][j]);
 	      // reads the last \n from the data matrix
@@ -617,13 +662,13 @@ read_uep_fromfile (FILE * uepfile, long tips, long nmlength, long **uep, long *u
 	  for (i = 0; i < tips; i++)
 	    {
 	      uep[i] = (long *) calloc (*uepsites, sizeof (long));
-	      uep[i+tips] = (long *) calloc (*uepsites, sizeof (long));
-	      FGETS (input, nmlength + 1, uepfile);  //reads each line
+	      uep[i + tips] = (long *) calloc (*uepsites, sizeof (long));
+	      FGETS (input, nmlength + 1, uepfile);	//reads each line
 	      for (j = 0; j < *uepsites; ++j)
 		fscanf (uepfile, "%li", &uep[i][j]);
 	      // finished reading first allele, no onto the second
 	      for (j = 0; j < *uepsites; ++j)
-		fscanf (uepfile, "%li", &uep[i+tips][j]);
+		fscanf (uepfile, "%li", &uep[i + tips][j]);
 	      // reads the last \n from the data matrix
 	      FGETS (input, LINESIZE, uepfile);
 	    }
@@ -633,7 +678,8 @@ read_uep_fromfile (FILE * uepfile, long tips, long nmlength, long **uep, long *u
 #endif
 
 void
-finish_read_seq (FILE * infile, data_fmt * data, option_fmt * options, long pop, long baseread)
+finish_read_seq (FILE * infile, data_fmt * data, option_fmt * options,
+		 long pop, long baseread)
 {
 
   long ind, baseread2 = 0, locus = 0;
@@ -643,7 +689,9 @@ finish_read_seq (FILE * infile, data_fmt * data, option_fmt * options, long pop,
 	{
 	  for (ind = 0; ind < data->numind[pop][FLOC]; ind++)
 	    {
-	      baseread2 = read_ind_seq (infile, data, options, locus, pop, ind, baseread);
+	      baseread2 =
+		read_ind_seq (infile, data, options, locus, pop, ind,
+			      baseread);
 	    }
 	  baseread = baseread2;
 	}
@@ -662,7 +710,9 @@ finish_read_seq (FILE * infile, data_fmt * data, option_fmt * options, long pop,
 	    {
 	      for (ind = 0; ind < data->numind[pop][FLOC]; ind++)
 		{
-		  baseread2 = read_ind_seq (infile, data, options, locus, pop, ind, baseread);
+		  baseread2 =
+		    read_ind_seq (infile, data, options, locus, pop, ind,
+				  baseread);
 		}
 	      baseread = baseread2;
 	    }
@@ -671,7 +721,8 @@ finish_read_seq (FILE * infile, data_fmt * data, option_fmt * options, long pop,
 }
 
 void
-print_data_summary (FILE * file, world_fmt * world, option_fmt * options, data_fmt * data)
+print_data_summary (FILE * file, world_fmt * world, option_fmt * options,
+		    data_fmt * data)
 {
   long total = 0;
   long pop;
@@ -679,29 +730,44 @@ print_data_summary (FILE * file, world_fmt * world, option_fmt * options, data_f
 
   fprintf (file, "Summary of data:\n");
   fprintf (file, "---------------\n");
-  switch(options->datatype)
+  switch (options->datatype)
     {
-    case 'a': strcpy(dstring,"Allelic data"); break;
-    case 'f': 
-    case 's': strcpy(dstring,"Sequence data"); break;
-    case 'b': 
-    case 'm': strcpy(dstring,"Microsatellite data"); break;
+    case 'a':
+      strcpy (dstring, "Allelic data");
+      break;
+    case 'f':
+    case 's':
+      strcpy (dstring, "Sequence data");
+      break;
+    case 'b':
+    case 'm':
+      strcpy (dstring, "Microsatellite data");
+      break;
     case 'n':
-    case 'u': strcpy(dstring,"Allelic data"); break;
-    default:  strcpy(dstring,"Unknown data [ERROR]"); break;
+    case 'u':
+      strcpy (dstring, "Allelic data");
+      break;
+    default:
+      strcpy (dstring, "Unknown data [ERROR]");
+      break;
     }
-  fprintf (file, "Datatype:                                %20s\n", 
-	   dstring);
-  fprintf (file, "Number of loci:                          %20li\n\n", data->loci);
+  fprintf (file, "Datatype:                                %20s\n", dstring);
+  fprintf (file, "Number of loci:                          %20li\n\n",
+	   data->loci);
 
-  fprintf (file, "Population                                        Individuals\n");
-  fprintf (file, "-------------------------------------------------------------\n");
+  fprintf (file,
+	   "Population                                        Individuals\n");
+  fprintf (file,
+	   "-------------------------------------------------------------\n");
   for (pop = 0; pop < world->numpop; pop++)
     {
-      fprintf (file, "%3li %-50.50s %6li\n", pop + 1, data->popnames[pop], data->numind[pop][FLOC]);
+      fprintf (file, "%3li %-50.50s %6li\n", pop + 1, data->popnames[pop],
+	       data->numind[pop][FLOC]);
       total += data->numind[pop][FLOC];
     }
-  fprintf (file, "Total of all populations                               %6li\n\n", total);
+  fprintf (file,
+	   "Total of all populations                               %6li\n\n",
+	   total);
   fflush (file);
 }
 
@@ -736,11 +802,14 @@ print_alleledata (world_fmt * world, data_fmt * data, option_fmt * options)
       print_header (world->outfile, pop, world, options, data);
       for (ind = 0; ind < data->numind[pop][FLOC]; ind++)
 	{
-	  fprintf (world->outfile, "%-*.*s ", (int) options->nmlength, (int) options->nmlength, data->indnames[pop][ind]);
+	  fprintf (world->outfile, "%-*.*s ", (int) options->nmlength,
+		   (int) options->nmlength, data->indnames[pop][ind]);
 	  mult80 = options->nmlength;
 	  for (locus = 0; locus < data->loci; locus++)
 	    {
-	      mult80 += 1 + strlen (data->yy[pop][ind][locus][0]) + strlen (data->yy[pop][ind][locus][1]);
+	      mult80 +=
+		1 + strlen (data->yy[pop][ind][locus][0]) +
+		strlen (data->yy[pop][ind][locus][1]);
 	      if (mult80 >= 80)
 		{
 		  mult80 = 0;
@@ -748,7 +817,9 @@ print_alleledata (world_fmt * world, data_fmt * data, option_fmt * options)
 		  for (i = 0; i < options->nmlength; i++)
 		    fputc (' ', world->outfile);
 		}
-	      fprintf (world->outfile, " %s.%-s", data->yy[pop][ind][locus][0], data->yy[pop][ind][locus][1]);
+	      fprintf (world->outfile, " %s.%-s",
+		       data->yy[pop][ind][locus][0],
+		       data->yy[pop][ind][locus][1]);
 	    }
 	  fprintf (world->outfile, "\n");
 	}
@@ -775,7 +846,8 @@ print_seqdata (world_fmt * world, option_fmt * options, data_fmt * data)
 }
 
 void
-print_header (FILE * outfile, long pop, world_fmt * world, option_fmt * options, data_fmt * data)
+print_header (FILE * outfile, long pop, world_fmt * world,
+	      option_fmt * options, data_fmt * data)
 {
   long i;
   long locus, mult80 = 80;
@@ -784,7 +856,7 @@ print_header (FILE * outfile, long pop, world_fmt * world, option_fmt * options,
   for (i = 0; i < (long) (80 - strlen (data->popnames[pop])); i++)
     fputc ('-', outfile);
   fprintf (outfile, "\n\n");
-  if (!strchr(SEQUENCETYPES,options->datatype))
+  if (!strchr (SEQUENCETYPES, options->datatype))
     {
       fprintf (outfile, "%-s  ", (data->loci == 1 ? "locus" : "loci "));
       for (i = 0; i < (options->nmlength - 6); i++)
@@ -800,7 +872,8 @@ print_header (FILE * outfile, long pop, world_fmt * world, option_fmt * options,
 	    }
 	  fprintf (outfile, "  %2li", locus + 1);
 	}
-      fprintf (outfile, "\n%-s\n", strncpy (input, "indiv.", options->nmlength));
+      fprintf (outfile, "\n%-s\n",
+	       strncpy (input, "indiv.", options->nmlength));
     }
 }
 
@@ -842,7 +915,8 @@ void
 addAllele (data_fmt * data, char s[], long locus, long *z)
 {
   long found = 0;
-  while ((data->allele[locus][found++][0] != '\0') && (strcmp (s, data->allele[locus][found - 1])))
+  while ((data->allele[locus][found++][0] != '\0')
+	 && (strcmp (s, data->allele[locus][found - 1])))
     ;
   if (found > (*z))
     {
@@ -866,7 +940,8 @@ set_numind (data_fmt * data)
 
 
 void
-print_seq_pop (long locus, long pop, world_fmt * world, option_fmt * options, data_fmt * data)
+print_seq_pop (long locus, long pop, world_fmt * world, option_fmt * options,
+	       data_fmt * data)
 {
   long ind;
   for (ind = 0; ind < data->numind[pop][locus]; ind++)
@@ -876,17 +951,21 @@ print_seq_pop (long locus, long pop, world_fmt * world, option_fmt * options, da
 }
 
 void
-print_seq_ind (long locus, long pop, long ind, world_fmt * world, option_fmt * options, data_fmt * data)
+print_seq_ind (long locus, long pop, long ind, world_fmt * world,
+	       option_fmt * options, data_fmt * data)
 {
   long site;
   char blank[2] = " ";
-  fprintf (world->outfile, "%-*.*s", (int) options->nmlength, (int) options->nmlength, data->indnames[pop][ind]);
+  fprintf (world->outfile, "%-*.*s", (int) options->nmlength,
+	   (int) options->nmlength, data->indnames[pop][ind]);
   fprintf (world->outfile, " %c", data->yy[pop][ind][locus][0][0]);
   for (site = 1; site < data->seq->sites[locus]; site++)
     {
       if ((site) % 60 == 0)
 	{
-	  fprintf (world->outfile, "\n%-*.*s %c", (int) options->nmlength, (int) options->nmlength, blank, data->yy[pop][ind][locus][0][site]);
+	  fprintf (world->outfile, "\n%-*.*s %c", (int) options->nmlength,
+		   (int) options->nmlength, blank,
+		   data->yy[pop][ind][locus][0][site]);
 	}
       else
 	{
@@ -902,12 +981,14 @@ print_seq_ind (long locus, long pop, long ind, world_fmt * world, option_fmt * o
 
 
 void
-print_locus_head (long locus, world_fmt * world, option_fmt * options, data_fmt * data)
+print_locus_head (long locus, world_fmt * world, option_fmt * options,
+		  data_fmt * data)
 {
   char *head;
   head = (char *) calloc (1, sizeof (char) * MAX (10, options->nmlength));
   sprintf (head, "Locus %li", locus);
-  fprintf (world->outfile, "%-*.*s --------10 --------20 --------30", (int) options->nmlength, (int) options->nmlength, head);
+  fprintf (world->outfile, "%-*.*s --------10 --------20 --------30",
+	   (int) options->nmlength, (int) options->nmlength, head);
   fprintf (world->outfile, " --------40 --------50 --------60\n");
 
   free (head);
@@ -931,12 +1012,17 @@ read_geofile (data_fmt * data, option_fmt * options, long numpop)
       data->ogeo[0] = (double *) calloc (1, sizeof (double) * numpop2);
       for (pop = 1; pop < numpop; pop++)
 	data->ogeo[pop] = data->ogeo[0] + numpop * pop;
-      read_distance_fromfile (data->geofile, numpop, options->nmlength, data->ogeo);
+      read_distance_fromfile (data->geofile, numpop, options->nmlength,
+			      data->ogeo);
       for (i = 0; i < numpop; i++)
 	for (j = i + 1; j < numpop; j++)
 	  {
-	    data->geo[mm2m (i, j, numpop)] = data->geo[mm2m (j, i, numpop)] = data->ogeo[i][j];
-	    data->lgeo[mm2m (i, j, numpop)] = data->lgeo[mm2m (j, i, numpop)] = data->ogeo[i][j] > 0.0 ? log (data->ogeo[i][j]) : -DBL_MAX;;
+
+	    data->geo[mm2m (i, j, numpop)] = data->geo[mm2m (j, i, numpop)] =
+	      1./data->ogeo[i][j];
+	    data->lgeo[mm2m (i, j, numpop)] =
+	      data->lgeo[mm2m (j, i, numpop)] =
+	      data->ogeo[i][j] > 0.0 ? log (1./data->ogeo[i][j]) : -DBL_MAX;;
 	  }
     }
 }
@@ -953,10 +1039,10 @@ read_uepfile (data_fmt * data, option_fmt * options, long numpop)
 
   for (i = 0; i < numpop; ++i)
     sumtips += data->numind[i][FLOC];
-  data->uep = (long **) calloc (number_genomes(options->datatype) * sumtips, 
+  data->uep = (long **) calloc (number_genomes (options->datatype) * sumtips,
 				sizeof (long *));
-  read_uep_fromfile (data->uepfile, sumtips, options->nmlength, data->uep, &data->uepsites, options->datatype);
+  read_uep_fromfile (data->uepfile, sumtips, options->nmlength, data->uep,
+		     &data->uepsites, options->datatype);
 }
 
 #endif
-
