@@ -17,12 +17,12 @@
   
 Copyright 2001 Peter Beerli and Joseph Felsenstein
 
-$Id: definitions.h,v 1.63 2001/07/16 16:03:21 beerli Exp $
+$Id: definitions.h,v 1.65 2001/09/07 23:56:13 beerli Exp $
 
   *----------------------------------------------------------------
   */
 #ifndef MIGRATEVERSION
-#define MIGRATEVERSION "1.2"
+#define MIGRATEVERSION "1.3"
 #endif
 
 #define MAINTAINER "Peter Beerli <beerli@genetics.washington.edu>"
@@ -77,6 +77,7 @@ $Id: definitions.h,v 1.63 2001/07/16 16:03:21 beerli Exp $
 #define GEOFILE "geofile"
 #define BOOTFILE "bootstrapfile"
 #define UEPFILE "uepfile"
+#define AICFILE "aicfile"
 #ifdef __STDIO_LOADED
 #define VMS
 #define MACHINE_TYPE "VAX/VMS C"
@@ -138,11 +139,32 @@ typedef unsigned char boolean;
 #define FIRST 0
 /* the first population is the panel for snps*/
 #define PANEL 0
-#define SEQUENCETYPES "snuf"  /*s=sequence,n=linked snp, u=unlinked snp, f=ancestral reconstruction*/
+#define SEQUENCETYPES "snuf"	/*s=sequence,n=linked snp, u=unlinked snp, f=ancestral reconstruction */
 #define SNPTYPES      "nu"
-#define ALLELETYPES   "amb" /*a=infinite allele,m=stepwise, b=brownian*/
+#define ALLELETYPES   "amb"	/*a=infinite allele,m=stepwise, b=brownian */
 
 #define CRLF "\r\n"
+
+#ifdef COMPAQ
+	#define EXP(a)  (((a)< -100) ? 0.0 : exp ((a)))
+#else
+	#define EXP(a)  exp ((a))
+#endif
+
+/* defines for speedier calc_like when we do NOT use undefined SLOWNET*/
+#ifdef SLOWNET
+#define CALCLIKE (*calc_like)
+#define CALCGRADIENT (*calc_gradient)
+#define SETUPPARAM0 (*setup_param0)
+#else
+#define CALCLIKE calc_loci_like
+#define CALCGRADIENT combine_gradient
+#ifdef MPI
+#define SETUPPARAM0 setup_parameter0_mpi
+#else
+#define SETUPPARAM0 setup_parameter0_standard
+#endif
+#endif
 /*=================================================================
   SPECIFIC FOR MIGRATE
 
@@ -155,7 +177,7 @@ typedef unsigned char boolean;
    MUST really be bigger than the number of 
    observed alleles, much bigger does not harm	*/
 #define MAXALLELES            93
-#define MAXBUFSIZE        100000      
+#define MAXBUFSIZE        100000
 #define LINESIZE            1024	/* setting this smaller can break */
 #define LONGLINESIZE       10000	/* setting this smaller can break */
 #define STRSIZE              255	/* setting this smaller can break */
@@ -171,7 +193,7 @@ typedef unsigned char boolean;
 #define SMALL_VALUE       10e-21
 #define EPSILON         0.000001	/* a small number */
 #define EPSILON4        0.000099	/* another small number */
-#define SMALLEPSILON       1e-15 	/* a smaller number */
+#define SMALLEPSILON       1e-15	/* a smaller number */
 #define BIGEPSILON         0.001	/* a not so small number */
 #define LONGCHAINEPSILON  10e100	/* an unreasonable big number, 
 					   so that the given number of 
@@ -204,8 +226,8 @@ typedef unsigned char boolean;
 #ifndef MAX
 #define MAX(a,b) (((a) > (b)) ? (a) : (b))
 #endif
-#define MIN3(a,b,c) (((a)<(b))&&((a)<(c)) ? (a) : (((b)<(c)) ? (b) : (c)))
-#define MAX3(a,b,c) (((a)>(b))&&((a)>(c)) ? (a) : (((b)>(c)) ? (b) : (c)))
+#define MYMIN3(a,b,c) (((a)<(b))&&((a)<(c)) ? (a) : (((b)<(c)) ? (b) : (c)))
+#define MYMAX3(a,b,c) (((a)>(b))&&((a)>(c)) ? (a) : (((b)>(c)) ? (b) : (c)))
 /*string conversion functions
   these replace all the atoi atol etc and use the strto[lif] function*/
 #define ATOL(a) strtol((a),(char **) NULL, 10)
@@ -218,7 +240,7 @@ typedef unsigned char boolean;
 #define BIGGEST_MIGRATION   10e9
 /* the migration limit is per population, should be plenty
    for moderate sample sizes */
-#define MIGRATION_LIMIT     1000
+#define MIGRATION_LIMIT     5000
 #define SMALLEST_PROB     1e-100
 #define SMALLEST_GAMMA      1e-3
 #define BIGGEST_GAMMA        1e9
@@ -263,9 +285,9 @@ typedef unsigned char boolean;
 #define CONTINUUM              4 /*- */
 #define NEIGHBOR               5 /*- */
 /* replication stuff */
-#define SINGLECHAIN            0 /*+ */
-#define MULTIPLECHAIN          1 /*+ */
-#define MULTIPLERUN            2 /*+ */
+#define SINGLECHAIN            0	/*+ */
+#define MULTIPLECHAIN          1	/*+ */
+#define MULTIPLERUN            2	/*+ */
 /* heating stuff */
 #define HEATED_CHAIN_NUM       4
 #define HEATED_THREAD_NUM      4
@@ -278,7 +300,7 @@ typedef unsigned char boolean;
 #define MULTILOCUS             1	/*+ */
 #define PROFILE                2	/*+ */
 /* print and plot material*/
-#define MAXPRINTVALLENGTH 16        /*+ maximal number of printpos*/
+#define MAXPRINTVALLENGTH 16	/*+ maximal number of printpos */
 #define PLOTALL                0	/*+ plot to outfile and mathfile */
 #define PLOTOUTFILE            1	/*+ plot only to outifle */
 #define PLOT4NM                0	/*+ use 4Nm instead of M=m/mu */
@@ -315,6 +337,7 @@ typedef unsigned char boolean;
 #define NOHEADER 0
 /* migration histogram material*/
 #define MIGHIST_ELEM 5
+#define MIGHIST_YSIZE 30
 /* stop while loops after some time*/
 #define PANIC_MAX           1000
 /* microsatellite stuff*/
@@ -330,9 +353,3 @@ typedef unsigned char boolean;
 #include <memdebug.h>
 #endif
 #endif /*definitions */
-
-
-
-
-
-
