@@ -8,10 +8,10 @@
 #include "combroyden.h"
 
 
-double gamma_loci_like (nr_fmt * nr, double denom, double *oparam,
-			double *olparam);
-double gamma_locus_like (nr_fmt * nr, double *oparam, double *olparam,
-			 long locus, double denom);
+double gamma_loci_like (nr_fmt * nr, double *oparam,
+			double *olparam, double denom);
+double gamma_locus_like (nr_fmt * nr, double *oparam,
+			double *olparam, double denom, long locus);
 double loggf (double r, double alpha, double theta1);
 void gamma_loci_derivative (helper_fmt * helper);
 void gamma_locus_derivative (helper_fmt * helper, long locus);
@@ -27,7 +27,7 @@ double dt_locus_gsum (long which, double x, double *param, nr_fmt * nr,
 
 
 double
-gamma_loci_like (nr_fmt * nr, double denom, double *oparam, double *olparam)
+gamma_loci_like (nr_fmt * nr, double *oparam, double *olparam, double denom)
 {
   long locus;
   double summ = 0.0;
@@ -36,16 +36,17 @@ gamma_loci_like (nr_fmt * nr, double denom, double *oparam, double *olparam)
       if (nr->skiploci[locus])
 	continue;
       nr->locilikes[locus] =
-	gamma_locus_like (nr, oparam, olparam, locus, denom);
+	gamma_locus_like(nr, oparam, olparam, denom, locus);
       summ += nr->locilikes[locus];
     }
   return summ;
 }
 
 double
-gamma_locus_like (nr_fmt * nr, double *oparam, double *olparam,
-		  long locus, double denom)
+gamma_locus_like (nr_fmt * nr, double *oparam, double *olparam, double denom,
+		  long locus)
 {
+//  nr_fmt *nr = helper->nr;
   long r;
   double tempmax = -DBL_MAX;
   double *temp;
@@ -58,7 +59,7 @@ gamma_locus_like (nr_fmt * nr, double *oparam, double *olparam,
       set_gamma_param (nr->param, oparam,
 		       nr->lparam, olparam, nr->rate[r], nr);
       temp[r] = nr->probcat[r] + loggf (nr->rate[r], alpha, theta1) - denom +
-	(*calc_like) (nr, nr->param, nr->lparam, locus);
+	calc_locus_like (nr, nr->param, nr->lparam, locus);
       //      printf("%f ",temp[r]);
       if (temp[r] > tempmax)
 	tempmax = temp[r];
@@ -92,14 +93,14 @@ gamma_loci_difference (helper_fmt * helper)
   memset (nr->d, 0, size * sizeof (double));
   setdoublevec1d (&lparam, helper->xv, size);
   setdoublevec1d (&param, helper->expxv, size);
-  mle = gamma_loci_like (nr, helper->weight, param, lparam);
+  mle = gamma_loci_like (nr, param, lparam, helper->weight);
   for (i = 0; i < size; i++)
     {
       memcpy (param, expxv, sizeof (double) * size);
       memcpy (lparam, xv, sizeof (double) * size);
       param1 = (param[i] += epsilon);
       lparam[i] = log (param[i]);
-      high = gamma_loci_like (nr, helper->weight, param, lparam);
+      high = gamma_loci_like (nr, param, lparam, helper->weight);
       param[i] -= 2. * epsilon;
       if (param[i] < 0.0)
 	{
@@ -109,7 +110,7 @@ gamma_loci_difference (helper_fmt * helper)
       else
 	{
 	  lparam[i] = log (param[i]);
-	  low = gamma_loci_like (nr, helper->weight, param, lparam);
+	  low = gamma_loci_like (nr, param, lparam, helper->weight);
 	}
       nr->d[i] = (high - low) / (param1 - param[i]);
     }
@@ -171,7 +172,7 @@ gamma_locus_derivative (helper_fmt * helper, long locus)
       lograte = log (rate);
       set_gamma_param (nr->param, helper->expxv,
 		       nr->lparam, helper->xv, rate, nr);
-      ll = nr->probcat[r] + (*calc_like) (nr, nr->param, nr->lparam, locus);
+      ll = nr->probcat[r] + calc_locus_like (nr, nr->param, nr->lparam, locus);
       ll += -rate * alpha / theta1 + lograte * (alpha - 1.) - denom;
       //summll[r] = ll;
       //if(ll>summllmax)
@@ -208,6 +209,7 @@ gamma_locus_derivative (helper_fmt * helper, long locus)
     {
       i = (nr->profilenum > 0) ? nr->indeks[z++] : ii;
       sumi = 0;
+
       for (r = 0; r < GAMMA_INTERVALS; r++)
 	sumi += ((double) stemp[i][r]) * EXP (temp[i][r] - tempmax[i]);
       if (i == 0)
