@@ -6,7 +6,7 @@ Peter Beerli 2000
 
 Copyright 2001 Peter Beerli and Joseph Felsenstein
 
-$Id: heating.c,v 1.8 2001/06/02 21:40:21 beerli Exp $
+$Id: heating.c,v 1.9 2001/07/25 19:27:23 beerli Exp $
 
 */
 #include "migration.h"
@@ -19,7 +19,8 @@ $Id: heating.c,v 1.8 2001/06/02 21:40:21 beerli Exp $
 
 
 void fill_tpool (tpool_t tpool, world_fmt ** universe, int universe_size);
-void tpool_init (tpool_t * tpoolp, int num_worker_threads, int max_queue_size, int do_not_block_when_full);
+void tpool_init (tpool_t * tpoolp, int num_worker_threads, int max_queue_size,
+		 int do_not_block_when_full);
 void thread_metropolize (void *world);
 int tpool_destroy (tpool_t tpool, int finish);
 void tpool_thread (tpool_t tpool);
@@ -27,17 +28,19 @@ int tpool_synchronize (tpool_t tpool, int finish);
 
 
 void
-tpool_init (tpool_t * tpoolp, int num_worker_threads, int max_queue_size, int do_not_block_when_full)
+tpool_init (tpool_t * tpoolp, int num_worker_threads, int max_queue_size,
+	    int do_not_block_when_full)
 {
   int i, rtn;
   tpool_t tpool;
 
-  tpool = (tpool_t) malloc (sizeof (struct tpool));
+  tpool = (tpool_t) malloc (sizeof (struct _tpool_t));
 
   tpool->num_threads = num_worker_threads;
   tpool->max_queue_size = max_queue_size;
   tpool->do_not_block_when_full = do_not_block_when_full;
-  tpool->threads = (pthread_t *) malloc (sizeof (pthread_t) * num_worker_threads);
+  tpool->threads =
+    (pthread_t *) malloc (sizeof (pthread_t) * num_worker_threads);
 
   tpool->cur_queue_size = 0;
   tpool->queue_head = NULL;
@@ -60,7 +63,9 @@ tpool_init (tpool_t * tpoolp, int num_worker_threads, int max_queue_size, int do
 
   for (i = 0; i < tpool->num_threads; i++)
     {
-      if ((rtn = pthread_create (&(tpool->threads[i]), NULL, (void *) tpool_thread, (void *) tpool)) != 0)
+      if ((rtn =
+	   pthread_create (&(tpool->threads[i]), NULL, (void *) tpool_thread,
+			   (void *) tpool)) != 0)
 	error ("Thread pool creation failed with pthread_create");
     }
   *tpoolp = tpool;
@@ -72,12 +77,14 @@ tpool_add_work (tpool_t tpool, void *routine, void *arg)
   tpool_work_t *workp;
   pthread_mutex_lock (&tpool->queue_lock);
 
-  if ((tpool->cur_queue_size == tpool->max_queue_size) && tpool->do_not_block_when_full)
+  if ((tpool->cur_queue_size == tpool->max_queue_size)
+      && tpool->do_not_block_when_full)
     {
       pthread_mutex_unlock (&tpool->queue_lock);
       return -1;
     }
-  while ((tpool->cur_queue_size == tpool->max_queue_size) && (!tpool->shutdown || tpool->queue_closed))
+  while ((tpool->cur_queue_size == tpool->max_queue_size)
+	 && (!tpool->shutdown || tpool->queue_closed))
     {
       pthread_cond_wait (&tpool->queue_not_full, &tpool->queue_lock);
     }
@@ -143,7 +150,8 @@ tpool_thread (tpool_t tpool)
       else
 	tpool->queue_head = myworkp->next;
 
-      if ((!tpool->do_not_block_when_full) && (tpool->cur_queue_size == (tpool->max_queue_size - 1)))
+      if ((!tpool->do_not_block_when_full)
+	  && (tpool->cur_queue_size == (tpool->max_queue_size - 1)))
 	pthread_cond_broadcast (&(tpool->queue_not_full));
 
       if (tpool->cur_queue_size == 0)
@@ -185,7 +193,9 @@ tpool_synchronize (tpool_t tpool, int fullnumber)
   while (tpool->done < fullnumber)
     {
       //      printf("(%i [%i]) ",tpool->done,   tpool->cur_queue_size);
-      if ((rtn = pthread_cond_wait (&(tpool->queue_done), &(tpool->queue_lock))) != 0)
+      if ((rtn =
+	   pthread_cond_wait (&(tpool->queue_done),
+			      &(tpool->queue_lock))) != 0)
 	error ("pthread_cond_wait failed in tpool_synchronize()");
     }
   tpool->done = 0;
@@ -217,7 +227,9 @@ tpool_destroy (tpool_t tpool, int finish)
     {
       while (tpool->cur_queue_size != 0)
 	{
-	  if ((rtn = pthread_cond_wait (&(tpool->queue_empty), &(tpool->queue_lock))) != 0)
+	  if ((rtn =
+	       pthread_cond_wait (&(tpool->queue_empty),
+				  &(tpool->queue_lock))) != 0)
 	    error ("pthread_cond_wait failed in tpool_destroy()");
 	}
     }
