@@ -3,7 +3,7 @@ Peter Beerli January 2000
 
 Copyright 2001 Peter Beerli and Joseph Felsenstein
 
-$Id: joint-chains.c,v 1.15 2001/07/25 19:27:24 beerli Exp $
+$Id: joint-chains.c,v 1.16 2001/09/07 23:56:13 beerli Exp $
 
 like(param) = Sum_g^G   ( p(g|param)/ L(param_j))
 
@@ -65,23 +65,19 @@ create_multiapg0 (double *apg0, nr_fmt * nr, long rep, long locus)
 void
 interpolate_like (nr_fmt * nr, long locus)
 {
-  static double *newlike, *lparam;
-  static double *diff, *oldiff;
-  static boolean done = FALSE, alldone;
+  double *newlike, *lparam;
+  double *diff, *oldiff;
+  boolean alldone;
   long j, z, r;
   long repdiff = nr->repstop - nr->repstart;
   double delta = 0.;
 
   boolean diffdone;
   double *oldlike = nr->world->chainlikes[locus];
-  if (!done)
-    {
-      newlike = (double *) calloc (repdiff, sizeof (double));
-      diff = (double *) calloc (repdiff, sizeof (double));
-      oldiff = (double *) calloc (repdiff, sizeof (double));
-      lparam = (double *) calloc (nr->numpop2, sizeof (double));
-      done = TRUE;
-    }
+  newlike = (double *) calloc (repdiff, sizeof (double));
+  diff = (double *) calloc (repdiff, sizeof (double));
+  oldiff = (double *) calloc (repdiff, sizeof (double));
+  lparam = (double *) calloc (nr->numpop2, sizeof (double));
   /* following material is reverse logistic regression 
      a la Geyer 1994 */
   z = 0;
@@ -138,4 +134,11 @@ interpolate_like (nr_fmt * nr, long locus)
 		  delta, z);
 	}
     }
+  free(newlike);
+  free(diff);
+  free(oldiff);
+  free(lparam);
 }
+
+
+
