@@ -17,7 +17,7 @@
 
 Copyright 2001 Peter Beerli and Joseph Felsenstein
 
-$Id: main.c,v 1.74 2001/07/02 21:30:30 beerli Exp $
+$Id: main.c,v 1.76 2001/09/07 23:56:13 beerli Exp $
 -------------------------------------------------------*/
 #include "migration.h"
 #include "definitions.h"
@@ -52,8 +52,20 @@ $Id: main.c,v 1.74 2001/07/02 21:30:30 beerli Exp $
 
 /* GLOBAL VARIABLES +++++++++++++++++++++++++++++++++++++*/
 int myID;			/* used if MPI is running otherwise == 0       */
+int allID;
+int color; /* either 1 or MPI_UNDEFINED */
 int numcpu;			/* used if MPI is running otherwise == 1     */
+int numlocicpu;
 int locidone;			/* used in MPI workers */
+//MPI_Comm       comm_all_worlds;
+#ifdef MPI
+MPI_Comm       comm_world;
+MPI_Comm comm_all_worlds;
+MPI_Comm comm_otherworld;
+#endif
+#ifdef SLOWNET
+int profiledone;  /* used in MPI workers in profiles on slow networks */
+#endif
 #ifdef PTHREADS
 tpool_t heating_pool;
 #else
@@ -64,8 +76,8 @@ tpool_t heating_pool;
 #endif
 
 char appl[20];
-long * seed;
-long * newseed;
+long *seed;
+long *newseed;
 
 int setup_locus (long locus, world_fmt * world, option_fmt * options,
 		 data_fmt * data);
@@ -87,10 +99,12 @@ void heating_prepare2 (world_fmt ** universe, int usize);
 long replicate_number (option_fmt * options, long chain, char type,
 		       long replicate);
 
-void combine_loci (char type, option_fmt * options, world_fmt * world, long *Gmax);
+void combine_loci (char type, option_fmt * options, world_fmt * world,
+		   long *Gmax);
 
-void run_sampler (option_fmt * options, data_fmt * data, world_fmt ** universe,
-		  int usize, long *outfilepos, long *Gmax);
+void run_sampler (option_fmt * options, data_fmt * data,
+		  world_fmt ** universe, int usize, long *outfilepos,
+		  long *Gmax);
 
 void run_loci (world_fmt ** universe, int usize, option_fmt * options,
 	       data_fmt * data, tpool_t * heating_pool, long maxreplicate,
@@ -148,6 +162,10 @@ void print_heating_progress2 (FILE * file, worldoption_fmt * options,
 			      double acc_h, double acc_w, double acc_c,
 			      long hot, long warm, long cold);
 
+
+void get_treedata (world_fmt * world, option_fmt * options);
+void get_mighistdata (world_fmt * world, option_fmt * options);
+
 /* Functions ++++++++++++++++++++++++++++++++++++++++++++++++*/
 /*--------------------------------------------
    main program 
@@ -166,17 +184,28 @@ main (int argc, char **argv)
   world_fmt **universe;
 #ifdef MPI
   //  long maxreplicate;
+  //printf("just started before init\n");
   MPI_Init (&argc, &argv);
-  MPI_Comm_size (MPI_COMM_WORLD, &numcpu);
-  MPI_Comm_rank (MPI_COMM_WORLD, &myID);
-  locidone = 0;
+//  comm_all_worlds = MPI_COMM_WORLD;
+//MPI_Comm_size (comm_all_worlds, &numcpu);
+//MPI_Comm_rank (comm_all_worlds, &allID);
+comm_world = MPI_COMM_WORLD;
+MPI_Comm_size (comm_world, &numcpu);
+MPI_Comm_rank (comm_world, &allID);
+
+myID = allID;
+#ifdef SLOWNET
+	profiledone=0;
+	which_calc_like(SINGLELOCUS);
+#endif
 #else
   myID = MASTER;
   numcpu = 1;
+  numlocicpu = 1;
 #endif
   // random number seed global inits
-  seed = malloc(sizeof(long) * 3);
-  newseed = malloc(sizeof(long) * 3);
+  seed = malloc (sizeof (long) * 3);
+  newseed = malloc (sizeof (long) * 3);
   // 
   universe = (world_fmt **) calloc (HEATED_CHAIN_NUM, sizeof (world_fmt *));
   this_string = (char *) calloc (1, sizeof (char) * 256);
@@ -206,8 +235,6 @@ main (int argc, char **argv)
   else
     {
       broadcast_options_worker (options);
-      //options->verbose=FALSE;
-      //      options->progress=FALSE;
       options->writelog = FALSE;
       options->menu = FALSE;
     }
@@ -236,7 +263,8 @@ main (int argc, char **argv)
     }
   // with MPI: the workers stay here much longer than the Master
   combine_loci (type, options, EARTH, &Gmax);	// multilocus MLE
-
+  // gather the treedata (1) into sumfile or from worker nodes
+  get_treedata (EARTH, options);
   /* printing of data ------------------------------------------- */
 #ifdef MPI
   if (myID == MASTER)
@@ -247,7 +275,7 @@ main (int argc, char **argv)
 #ifdef MPI
     }
 #endif
-  if(options->aic || (EARTH->options->lratio->counter > 0))
+  if (options->aic || (EARTH->options->lratio->counter > 0))
     {
 #ifdef MPI
       if (myID == MASTER)
@@ -255,40 +283,46 @@ main (int argc, char **argv)
 #endif
 	  if (EARTH->options->lratio->counter > 0)
 	    print_lratio_test (EARTH, &Gmax);
-	  if(options->aic)
-	    akaike_information(EARTH, &Gmax);
+	  if (options->aic)
+	    akaike_information (EARTH, &Gmax);
 #ifdef MPI
 	}
 #endif
     }
-  print_mighist (EARTH);      
+  if (myID == MASTER)
+    {
+#ifdef MPI
+      get_mighistdata (EARTH, options);
+#endif
+      print_mighist (EARTH);
+    }
   /* print UEP probabilities ------------------------------------ */
 #ifdef UEP
   if (options->uep)
     analyze_uep (EARTH);
 #endif
   /* profile tables --------------------------------------------- */
-#ifdef SLOWNET /*DOES NOT WORK YET*/
+#ifdef SLOWNET			/*DOES NOT WORK YET */
   // if there are many loci but only few parameters to estimate
   // this will be still slow and saturating the network
-  // may still be te better solution
-  if(myID == MASTER)
-    {
-      mpi_send_stop (EARTH); // stop workers
-    }
-  distribute_treedata(EARTH); //distributes to all numcpu all the treesummaries
-  profile_tables (options, EARTH, &Gmax); 
-#else
-#ifdef MPI  /*WORKS*/
+  // with parameter requests may still be the better solution
+  which_calc_like(PROFILE);
   if (myID == MASTER)
     {
+      mpi_send_stop (EARTH);	// stop workers
+    }
+  profile_tables (options, EARTH, &Gmax);
+#else
+#ifdef MPI /*WORKS*/
+    if (myID == MASTER)
+    {
 #endif
       profile_tables (options, EARTH, &Gmax);
 #ifdef MPI
     }
-#endif 
+#endif
 #endif /*SLOWNET*/
-  if (myID == MASTER)
+    if (myID == MASTER)
     {
       print_finish (EARTH, outfilepos);
       /* closing all files ------------------------------------------ */
@@ -299,11 +333,13 @@ main (int argc, char **argv)
     }
 #ifdef MPI
 #ifndef SLOWNET
-  mpi_send_stop (EARTH); // stop workers
+  mpi_send_stop (EARTH);	// stop workers
+  //printf("%i> at barrier in slownet-main before finalize\n",myID);
+  MPI_Barrier(comm_all_worlds);
 #endif
   MPI_Finalize ();
 #endif
-  apocalypse_now(EARTH);
+  apocalypse_now (EARTH);
   return 0;
 }				/* main end */
 
@@ -313,32 +349,32 @@ void
 combine_loci (char type, option_fmt * options, world_fmt * world, long *Gmax)
 {
   long kind = MULTILOCUS;
-  if(options->readsum)
+  if (options->readsum)
     {
-      if(world->loci==1)
-	kind=SINGLELOCUS;
+      if (world->loci == 1)
+	kind = SINGLELOCUS;
     }
   if ((world->loci - world->skipped > 1) || (options->readsum))
     {
-      if(options->gamma)
+      if (options->gamma)
 	world->param0[world->numpop2] = world->options->alphavalue;
       world->repkind = set_repkind (options);
 #ifdef MPI
       if (myID == MASTER)
 	{
-	  mpi_gmax_master(world, Gmax);
+	  mpi_gmax_master (world, Gmax);
 #endif
+	decide_plot (world->options, world->options->lchains, world->options->lchains, 'l');
 	  estimateParameter (options->replicatenum, *Gmax, world,
 			     options, world->cov[world->loci], 0 /*chain */ ,
-			     type, kind,
-			     world->repkind);	
+			     type, kind, world->repkind);
 #ifdef MPI
 	}
       else
 	{
-	  printf("%i> before gmax worker\n",myID);
+	  //printf ("%i> before gmax worker\n", myID);
 	  mpi_gmax_worker (world);
-	  printf("%i> start worker postlike calculations\n",myID);
+	  //printf ("%i> start worker postlike calculations\n", myID);
 	  mpi_maximize_worker (world, options->replicatenum);
 	}
 #endif
@@ -353,7 +389,14 @@ run_sampler (option_fmt * options, data_fmt * data, world_fmt ** universe,
   long maxreplicate;
   long treefilepos;
   char *this_string;
-
+#ifdef MPI
+//int *ranks;
+//MPI_Group worldgroup, locigroup;
+ if(myID < data->loci+1)
+	color=1;
+else
+	color = MPI_UNDEFINED;
+#endif
   this_string = (char *) calloc (1024, sizeof (char));
 #ifdef MPI
   if (myID == MASTER)
@@ -361,19 +404,19 @@ run_sampler (option_fmt * options, data_fmt * data, world_fmt ** universe,
 #endif
       get_data (data->infile, data, options);
 #ifdef MPI
-      numcpu = MIN(numcpu,data->loci+1);//manip global 
       //broadcast_data_master(data,options);
-      if(numcpu==1)
+      if (numcpu == 1)
 	{
-	  error("This program was compiled to use a parallel computer\n and you tried to run it on only a single node.\nThis will not work because it uses a \n\"single_master-many_worker\" architecture \nand needs at least TWO nodes and TWO loci\n");
+	  error
+	    ("This program was compiled to use a parallel computer\n and you tried to run it on only a single node.\nThis will not work because it uses a \n\"single_master-many_worker\" architecture \nand needs at least TWO nodes and TWO loci\n");
 	}
     }
   else
     {
-      //      printf("%i> data=%li\n", myID, data);
       get_data (data->infile, data, options);
-    }
-  //broadcast_data_worker(data,options);
+    }		
+     MPI_Comm_dup(comm_world, &comm_all_worlds);
+	MPI_Comm_rank(comm_world, &myID);
 #endif
   set_plot (options);
   init_world (EARTH, data, options);
@@ -408,26 +451,20 @@ run_sampler (option_fmt * options, data_fmt * data, world_fmt ** universe,
     }
   else
     {
-      mpi_runloci_worker (universe, usize, options, data,
-			 &heating_pool, maxreplicate, &treefilepos, Gmax);
+		mpi_runloci_worker (universe, usize, options, data,
+			  &heating_pool, maxreplicate, &treefilepos, Gmax);
     }
-  //  printf ("%i>         All loci run\n",myID);
 #else
   run_loci (universe, usize, options, data,
 	    &heating_pool, maxreplicate, &treefilepos, Gmax);
 #endif
 
-  /* save all genealogy-summaries --------------------------- */
-  if (myID==MASTER && options->writesum)
-    {
-      write_savesum (EARTH);
-    }
 #ifdef PTHREADS
   tpool_destroy (heating_pool, 1);
 #endif
-  if(options->heating)
+  if (options->heating)
     {
-      for(i=0;i<options->heated_chains;i++)
+      for (i = 0; i < options->heated_chains; i++)
 	{
 	  free_tree (universe[i]->root, universe[i]);
 	  free_timevector (universe[i]->treetimes);
@@ -494,7 +531,6 @@ run_locus (world_fmt ** universe, int usize, option_fmt * options,
 		print_tree (EARTH, 0, treefilepos);
 	    }
 	  EARTH->chains = chains;
-	  //      EARTH->lsteps = log (oldsteps);
 	  /* loop over chains--------------------------- */
 	  for (chain = 0;
 	       chain < chains || (type == 'l' && chain >= chains
@@ -512,8 +548,6 @@ run_locus (world_fmt ** universe, int usize, option_fmt * options,
 	      EARTH->maxdatallike = EARTH->likelihood[0];
 	      if (options->heating)
 		heating_prepare2 (universe, usize);
-	      //free(options);
-	      //free(data);
 	      if (type == 'l')
 		{
 		  if (chain >= chains - 1)
@@ -557,8 +591,8 @@ run_locus (world_fmt ** universe, int usize, option_fmt * options,
 			    update_uepanc (EARTH);
 #endif
 			  heated_swap (universe, EARTH->options, &acc_vh,
-				       &acc_h, &acc_w, &acc_c, &vh, &h, &w, &jj,
-				       &j, &hot, &warm, &cold);
+				       &acc_h, &acc_w, &acc_c, &vh, &h, &w,
+				       &jj, &j, &hot, &warm, &cold);
 			}
 		      else
 			{
@@ -573,11 +607,6 @@ run_locus (world_fmt ** universe, int usize, option_fmt * options,
 			  EARTH->maxdatallike = EARTH->likelihood[EARTH->G];
 			}
 		    }
-		  /*if(options->verbose && !options->heating)
-		     {
-		     printf("Cold:%5.5f\n",
-		     EARTH->likelihood[EARTH->G]);
-		     } */
 		  if (options->heating)
 		    {
 		      advance_clone_like (VENUS, w, &VENUS->G);
@@ -585,7 +614,8 @@ run_locus (world_fmt ** universe, int usize, option_fmt * options,
 		      advance_clone_like (SUN, vh, &SUN->G);
 		      for (ii = HEATED_CHAIN_NUM;
 			   ii < EARTH->options->heated_chains; ++ii)
-			advance_clone_like (universe[ii], universe[ii]->accept,
+			advance_clone_like (universe[ii],
+					    universe[ii]->accept,
 					    &(universe[ii])->G);
 		    }
 		  /* store condensed tree information
@@ -593,8 +623,8 @@ run_locus (world_fmt ** universe, int usize, option_fmt * options,
 #ifdef UEP
 		  store_uep (EARTH);
 #endif
-		  condense_time (EARTH, &step, &j, &accepted, &EARTH->G, &steps,
-				 oldsteps, rep);
+		  condense_time (EARTH, &step, &j, &accepted, &EARTH->G,
+				 &steps, oldsteps, rep);
 		  if (step > 0)
 		    advance_clone_like (EARTH, j, &EARTH->G);
 		}		// end steps
@@ -605,13 +635,11 @@ run_locus (world_fmt ** universe, int usize, option_fmt * options,
 		      sizeof (double) * EARTH->numpop2);
 	      log_param0 (EARTH->atl[rep][locus].param0,
 			  EARTH->atl[rep][locus].lparam0, EARTH->numpop2);
-	      copies2lcopies(&EARTH->atl[rep][locus]);
+	      copies2lcopies (&EARTH->atl[rep][locus]);
 	      EARTH->repkind = SINGLECHAIN;
 	      // single chain,  single replicate
-	      // estimateParameter(locus, rep, chain, type, 
-	      //                        EARTH, SINGLELOCUS);
 	      kind = SINGLELOCUS;
-	      *Gmax = (EARTH->G > *Gmax) ? EARTH->G+1 : (*Gmax) ;
+	      *Gmax = (EARTH->G > *Gmax) ? EARTH->G + 1 : (*Gmax);
 	      estimateParameter (rep, *Gmax, EARTH, options,
 				 EARTH->cov[locus], chain, type, kind,
 				 EARTH->repkind);
@@ -623,16 +651,15 @@ run_locus (world_fmt ** universe, int usize, option_fmt * options,
 				      increment, &acc_vh, &acc_h, &acc_w,
 				      &acc_c, &hot, &warm, &cold);
 	    }			//end chains
-	  change_chaintype (locus, &type, EARTH, &increment, &oldsteps, &chains,
-			    options);
+	  change_chaintype (locus, &type, EARTH, &increment, &oldsteps,
+			    &chains, options);
 	  /* evaluate multiple long chain estimates */
 	  if (runs < 0 && options->replicate && options->replicatenum == 0)
 	    {
 	      EARTH->repkind = MULTIPLECHAIN;
 	      estimateParameter (rep, *Gmax, EARTH, options,
 				 EARTH->cov[locus], chain, type,
-				 SINGLELOCUS,
-				 EARTH->repkind);
+				 SINGLELOCUS, EARTH->repkind);
 	    }
 	}			//end runs 
     }				//end replicates
@@ -649,6 +676,58 @@ run_locus (world_fmt ** universe, int usize, option_fmt * options,
     }
 }
 
+ /* save all genealogy-summaries --------------------------- */
+void
+get_treedata (world_fmt * world, option_fmt * options)
+{
+#ifdef MPI
+  long maxreplicate = (options->replicate
+		       && options->replicatenum >
+		       0) ? options->replicatenum : 1;
+#endif
+  if (myID == MASTER)
+    {
+      if (options->writesum)
+	{
+#ifdef MPI
+	  //get all sumfile data from the workers using unpack_result_buffer()
+	  mpi_results_master (MIGMPI_SUMFILE, world, maxreplicate,
+			      unpack_sumfile_buffer);
+#endif
+	  write_savesum (world);
+	}
+#ifdef SLOWNET
+      else
+	{
+	  mpi_results_master (MIGMPI_SUMFILE, world, maxreplicate,
+			      unpack_sumfile_buffer);
+	}
+#endif
+    }
+}
+
+ /* get all mig-histogram data --------------------------- */
+void
+get_mighistdata (world_fmt * world, option_fmt * options)
+{
+#ifdef MPI
+  long maxreplicate = (options->replicate
+		       && options->replicatenum >
+		       0) ? options->replicatenum : 1;
+#endif
+  if (myID == MASTER)
+    {
+      if (options->mighist)
+	{
+#ifdef MPI
+	  //get all mighist data from the workers using unpack_mighist_buffer()
+	  mpi_results_master (MIGMPI_MIGHIST, world, maxreplicate,
+			      unpack_mighist_buffer);
+#endif
+	}
+    }
+}
+
 void
 heated_swap (world_fmt ** universe, worldoption_fmt * options, double *acc_vh,
 	     double *acc_h, double *acc_w, double *acc_c, long *vh, long *h,
@@ -746,9 +825,9 @@ change_chaintype (long locus, char *type, world_fmt * world, long *increment,
 }
 
 void
-prepare_next_chain (world_fmt ** universe, worldoption_fmt * options, char type,
-		    long chain, long *chains, long *pluschain, long locus,
-		    long replicate)
+prepare_next_chain (world_fmt ** universe, worldoption_fmt * options,
+		    char type, long chain, long *chains, long *pluschain,
+		    long locus, long replicate)
 {
   long i;
   EARTH->likelihood[0] = EARTH->likelihood[EARTH->G];
@@ -783,8 +862,8 @@ prepare_next_chain (world_fmt ** universe, worldoption_fmt * options, char type,
 void
 print_heating_progress (world_fmt ** universe, worldoption_fmt * options,
 			long steps, long increment, double *acc_vh,
-			double *acc_h, double *acc_w, double *acc_c, long *hot,
-			long *warm, long *cold)
+			double *acc_h, double *acc_w, double *acc_c,
+			long *hot, long *warm, long *cold)
 {
   if (options->heating)
     {
@@ -829,7 +908,10 @@ long
 analyze_olddata (world_fmt * world,
 		 option_fmt * options, data_fmt * data, long *outfilepos)
 {
-  long Gmax=0;
+#ifndef MPI
+  long locus;
+#endif
+  long Gmax = 0;
   Gmax = read_savesum (world, options, data);
   synchronize_param (world, options);
   if (options->plot)
@@ -838,19 +920,31 @@ analyze_olddata (world_fmt * world,
       world->options->plotnow = TRUE;
     }
   world->locus = 0;
-  if(myID==MASTER)
+  if (myID == MASTER)
     {
       *outfilepos = print_title (world, options);
       print_options (stdout, world, options, data);
       print_options (world->outfile, world, options, data);
 #ifdef MPI
-      mpi_runloci_master(world->loci, world->who);
+      mpi_runloci_master (world->loci, world->who);
+#else
+      for(locus=0;locus<world->loci;locus++)
+	{
+	  if (options->replicate && options->replicatenum > 0)
+	    {
+	      world->locus=locus;
+	      world->repkind = MULTIPLERUN;
+	      estimateParameter (options->replicatenum, Gmax, world, options,
+				 world->cov[locus], 1, 'l',
+				 SINGLELOCUS, world->repkind);
+	    }
+	}
 #endif
     }
 #ifdef MPI
   else
     {
-      assignloci_worker(world);
+      assignloci_worker (world);
     }
 #endif
   return Gmax;
@@ -859,33 +953,44 @@ analyze_olddata (world_fmt * world,
 void
 profile_tables (option_fmt * options, world_fmt * world, long *gmaxptr)
 {
+#ifndef SLOWNET
   long i;
+#endif
   if (options->profile != NONE)
     {
-      PAGEFEEDWORLD;
+	world->buffer = (char *) realloc(world->buffer, LINESIZE*100 * sizeof(char));
       if (options->printprofsummary)
 	allocate_profile_percentiles (world);
 #ifdef HAVE_STRFTIME
       time (&world->starttime);
 #endif
+#ifdef SLOWNET
+	mpi_broadcast_results(world);
+	if(myID==MASTER)
+	{
+		print_profile_title(world);
+		mpi_profiles_master(world, world->numpop2 + (world->options->gamma ? 1 : 0), world->profilewho);
+	}
+	else
+		mpi_profiles_worker(world, gmaxptr);
+#else
+	print_profile_title(world);
       for (i = 0; i < world->numpop2 + (world->options->gamma ? 1 : 0); i++)
 	{
-	  if (print_profile_likelihood_driver (i, world, gmaxptr))
-	    {
-	      if ((i + 1) % 2 == 0)
-		PAGEFEEDWORLD;
-	      fflush (world->outfile);
-	    }
+		print_profile_likelihood_driver (i, world, gmaxptr);
+		fprintf(world->outfile,"%s\n\n",world->buffer);
+		world->buffer[0] = '\0';
 	}
-#ifdef MPI
-      printf("will print percentile tables\n");
-#endif
-      if (options->printprofsummary)
+#endif			
+    if (myID==MASTER && options->printprofsummary)
 	{
 	  print_profile_percentile (world);
+	  fprintf(world->outfile,"%s\n\n",world->buffer);
+		world->buffer[0] = '\0';	  
 	  destroy_profile_percentiles (world);
 	}
     }
+	//free(world->buffer);
 }
 
 void
@@ -922,13 +1027,14 @@ setup_locus (long locus, world_fmt * world, option_fmt * options,
 	  data->seq->endsite += data->seq->addon;
 	}
     }
-  if (!options->replicate || (options->replicate && options->replicatenum == 0)
+  if (!options->replicate
+      || (options->replicate && options->replicatenum == 0)
       || (options->replicate && world->replicate == options->replicatenum))
     {
       //free_datapart (data, options, locus);
       if (data->skiploci[locus])
 	{			/* skip loci with no tips */
-	  //	  cleanup_world (world, locus);
+	  //      cleanup_world (world, locus);
 	  return 0;
 	}
     }
@@ -958,8 +1064,8 @@ setup_locus (long locus, world_fmt * world, option_fmt * options,
 }
 
 void
-condense_time (world_fmt * world, long *step, long *j, long *accepted, long *G,
-	       long *steps, long oldsteps, long rep)
+condense_time (world_fmt * world, long *step, long *j, long *accepted,
+	       long *G, long *steps, long oldsteps, long rep)
 {
   world->rep = rep;
   if (*step == 0)
