@@ -5,15 +5,19 @@
    
 Copyright 2001 Peter Beerli and Joseph Felsenstein
 
-$Id: migrate_mpi.c,v 1.10 2001/07/25 19:27:24 beerli Exp $
+$Id: migrate_mpi.c,v 1.11 2001/09/07 23:56:13 beerli Exp $
 */
 #ifdef MPI
 #include "migrate_mpi.h"
 #include "broyden.h"
 #include "combroyden.h"
 #include "gammalike.h"
+#include "profile.h"
 #include "options.h"
 #include "world.h"
+#include "joint-chains.h"
+/*should go into profile.h*/
+#define GRIDSIZE 9
 
 extern void run_locus (world_fmt ** universe, int usize,
 		       option_fmt * options, data_fmt * data,
@@ -28,17 +32,14 @@ void mpi_results_master (double sendtype, world_fmt * world,
 					 long locus, long maxrep,
 					 long numpop));
 
-void mpi_results_worker (long bufsize, world_fmt * world,
+void mpi_results_worker (long bufs, world_fmt * world,
 			 long maxrep,
 			 long (*pack) (char **buffer, world_fmt * world,
 				       long locus, long maxrep, long numpop));
 void assignloci_worker (world_fmt * world);
 void swap_atl (long from, long to, world_fmt * world);
-
-/* global variable pack points to function pack_stuff() */
-//static void (*pack) (node *, world_fmt *, long);
-/* global variable unpack points to function unpack_stuff() */
-//static void (*unpack) (node *, world_fmt *, long);
+long pack_quantile(char **buffer, quantile_fmt quant, long n);
+void unpack_quantile(char *buffer, quantile_fmt quant, long n);
 
 void
 mpi_runloci_master (long loci, int *who)
@@ -50,26 +51,30 @@ mpi_runloci_master (long loci, int *who)
   long numsent = 0;
   for (locus = 0; locus < MIN (loci, numcpu - 1); locus++)
     {
-      MPI_Send (&locus, 1, MPI_LONG, locus + 1, locus + 1, MPI_COMM_WORLD);
+      MPI_Send (&locus, 1, MPI_LONG, locus + 1, locus + 1, comm_world);
       numsent++;
     }
   for (locus = 0; locus < loci; locus++)
     {
       MPI_Recv (&locusdone, 1, MPI_LONG, MPI_ANY_SOURCE, MPI_ANY_TAG,
-		MPI_COMM_WORLD, &status);
+		comm_world, &status);
       sender = status.MPI_SOURCE;
       who[locusdone] = sender;
       if (numsent < loci)
 	{
 	  MPI_Send (&numsent, 1, MPI_LONG, sender, numsent + 1,
-		    MPI_COMM_WORLD);
+		    comm_world);
 	  numsent++;
 	}
       else
 	{
-	  MPI_Send (&loci, 1, MPI_LONG, sender, 0, MPI_COMM_WORLD);	//end of loci
+	  MPI_Send (&loci, 1, MPI_LONG, sender, 0, comm_world);	//stop worker do wait for new loci
 	}
     }
+    for(sender=MIN (loci, numcpu - 1)+1; sender < numcpu; sender++)
+	{
+		MPI_Send (&loci, 1, MPI_LONG, sender, 0, comm_world);	//stop worker do wait for new loci
+	}
 }
 
 
@@ -85,12 +90,12 @@ mpi_runloci_worker (world_fmt ** universe, int usize,
   while (!done)
     {
       MPI_Recv (&locus, 1, MPI_LONG, MASTER, MPI_ANY_TAG,
-		MPI_COMM_WORLD, &status);
+		comm_world, &status);
       if (status.MPI_TAG != 0)	//stop condition
 	{
 	  run_locus (universe, usize, options, data,
 		     heating_pool, maxreplicate, locus, treefilepos, Gmax);
-	  MPI_Send (&locus, 1, MPI_LONG, MASTER, locus + 1, MPI_COMM_WORLD);
+	  MPI_Send (&locus, 1, MPI_LONG, MASTER, locus + 1, comm_world);
 	  /* we want to know what locus we worked for
 	     - to control the work sent by master
 	     - to use in setup_parameter0() [combroyden2.c] */
@@ -124,23 +129,21 @@ mpi_likelihood_master (double *param, double *lparam,
   memcpy (temp + 1, param, (numelem - 1) * sizeof (double));
   memcpy (temp + numelem, lparam, (numelem - 1) * sizeof (double));
   memset (nr->locilikes, 0, sizeof (double) * world->loci);
-  for (worker = 1; worker < numcpu; worker++)
+  for (worker = 1; worker < MIN(world->loci+1,numcpu); worker++)
     {
       MPI_Send (temp, (int) numelem2, MPI_DOUBLE, worker,
-		worker, MPI_COMM_WORLD);
+		worker, comm_world);
     }
-  for (worker = 1; worker < numcpu; worker++)
+  for (worker = 1; worker < MIN(world->loci+1,numcpu); worker++)
     {
-      //      printf("%i> wait for like calc\n",myID); fflush(stdout);
       MPI_Recv (tmp, (int) world->loci, MPI_DOUBLE, MPI_ANY_SOURCE,
-		MPI_ANY_TAG, MPI_COMM_WORLD, &status);
+		MPI_ANY_TAG, comm_world, &status);
       sender = status.MPI_SOURCE;
       tag = status.MPI_TAG;
-      //      printf("%i> like calc got %i from %i\n",myID, sender, tag);
       // the worker send a vector of values 
       // e.g. (0,0,0,-12,0,-34,0,0,0), these are all loci
       // of which most of them were not evaluated
-      // the loop updates the master copy of locilikes
+      // t he loop updates the master copy of locilikes
       for (locus = 0; locus < world->loci; locus++)
 	{
 	  nr->locilikes[locus] += tmp[locus];
@@ -151,7 +154,6 @@ mpi_likelihood_master (double *param, double *lparam,
       logres += nr->locilikes[locus];
     }
   nr->llike = logres;
-  //  printf("%i> Log(L) = %f -------------------------\n",myID, logres);
   free (temp);
   return logres;
 }
@@ -168,17 +170,14 @@ mpi_likelihood_worker (world_fmt * world, helper_fmt * helper, long rep)
       locus = nr->world->who[ww];
       if (!world->options->gamma)
 	{
-	  //      printf("%i> in mpi_likelihood_worker\n",myID);
-	  nr->locilikes[locus] =
-	    (*calc_like) (nr, nr->param, nr->lparam, locus);
-	  //      printf("%i> locilikes[locus=%li]=%f\n", myID, locus, nr->locilikes[locus]);
+	  nr->locilikes[locus] = calc_locus_like (nr, nr->param, nr->lparam, locus);
 	}
       else
 	{
 	  helper->locus = locus;
 	  helper->nr->locilikes[locus] = gamma_locus_like (nr,
 							   helper->expxv,
-							   helper->xv, locus);
+							   helper->xv, helper->weight, locus);
 	}
     }
 }
@@ -192,25 +191,19 @@ mpi_gmax_master (world_fmt * world, long *Gmax)
   long tmp;
   int tag;
   int numreceived = 0;
-  //  printf("%i> waiting for gmax\n",myID); 
   *Gmax = 0.;
-  MPI_Bcast (Gmax, 1, MPI_LONG, MASTER, MPI_COMM_WORLD);
+  MPI_Bcast (Gmax, 1, MPI_LONG, MASTER, comm_world);
   while (numreceived < numcpu - 1)
     {
-      //      printf("%i> waiting for gmax in loop %i\n",myID, numreceived); 
       MPI_Recv (&tmp, 1, MPI_LONG, MPI_ANY_SOURCE,
-		MPI_ANY_TAG, MPI_COMM_WORLD, &status);
+		MPI_ANY_TAG, comm_world, &status);
       sender = status.MPI_SOURCE;
       tag = status.MPI_TAG;
-      //      printf("%i> received gmax=%li from %li with tag %i\n",myID, 
-      //             tmp, sender,tag); 
-      fflush (stdout);
       if (*Gmax < tmp)
 	*Gmax = tmp;
       numreceived++;
     }
-//  printf("%i> broadcast gmax=%li\n",myID,*Gmax); 
-  MPI_Bcast (Gmax, 1, MPI_LONG, MASTER, MPI_COMM_WORLD);
+  MPI_Bcast (Gmax, 1, MPI_LONG, MASTER, comm_world);
 }
 
 void
@@ -221,9 +214,8 @@ mpi_gmax_worker (world_fmt * world)
   long repstop;
   long r;
   long locus;
-  long Gmax = 1000000;
-//  printf("%i> before bcast gmax in worker\n",myID);fflush(stdout); 
-  MPI_Bcast (&Gmax, 1, MPI_LONG, MASTER, MPI_COMM_WORLD);
+  long Gmax = 1;
+  MPI_Bcast (&Gmax, 1, MPI_LONG, MASTER, comm_world);
   set_replicates (world, world->repkind, world->options->replicatenum,
 		  &repstart, &repstop);
 
@@ -236,9 +228,7 @@ mpi_gmax_worker (world_fmt * world)
 	    Gmax = world->atl[r][locus].T;
 	}
     }
-//  printf("%i> will send gmax=%li for locus %li\n",myID,Gmax,locus+1); 
-//  fflush(stdout);
-  MPI_Send (&Gmax, 1, MPI_LONG, MASTER, myID, MPI_COMM_WORLD);
+  MPI_Send (&Gmax, 1, MPI_LONG, MASTER, myID, comm_world);
 }
 
 
@@ -249,20 +239,27 @@ mpi_send_stop (world_fmt * world)
   double *temp;
   long numelem = world->numpop2 + (world->options->gamma ? 2 : 1);
   long numelem2 = 2 * numelem;
-  //  printf("%i> will send stop signal\n",myID);fflush(stdout);
   temp = (double *) calloc (numelem2, sizeof (double));
   temp[0] = MIGMPI_END;
   for (worker = 1; worker < numcpu; worker++)
     {
-      //      printf("%i> stop signal: numcpu=%i, loci=%li, numpop2=%li\n",myID, 
-      //     numcpu, world->loci,numelem2);fflush(stdout);
-      MPI_Send (temp, (int) numelem2, MPI_DOUBLE, worker, 0, MPI_COMM_WORLD);	//end of loci
-      //printf("%i> sent stop signal %f to worker %li\n",myID,
-      //temp[0], worker);
+      MPI_Send (temp, (int) numelem2, MPI_DOUBLE, worker, 0, comm_world);	//end of loci
     }
   free (temp);
 }
 
+void
+mpi_send_stop_tag (int worker, world_fmt * world)
+{
+  double *temp;
+  long numelem = world->numpop2 + (world->options->gamma ? 2 : 1);
+  long numelem2 = 2 * numelem;
+  temp = (double *) calloc (numelem2, sizeof (double));
+  temp[0] = MIGMPI_END;
+  MPI_Send (temp, (int) numelem2, MPI_DOUBLE, worker, 0, comm_world);	//end of loci
+  free (temp);
+}
+
 void
 mpi_results_stop (void)
 {
@@ -270,9 +267,7 @@ mpi_results_stop (void)
   long dummy = 0;
   for (worker = 1; worker < numcpu; worker++)
     {
-      //printf("%i> will send stop\n", myID);
-      MPI_Send (&dummy, 1, MPI_LONG, worker, 0, MPI_COMM_WORLD);
-      //printf("%i> have sent stop to worker %i\n", myID, worker);
+      MPI_Send (&dummy, 1, MPI_LONG, worker, 0, comm_world);
     }
 }
 
@@ -291,24 +286,23 @@ mpi_gradient_master (nr_fmt * nr, world_fmt * world, int *who)
   temp = (double *) calloc (numelem2, sizeof (double));
   tempindex = (long *) calloc (numelem, sizeof (long));
   temp[0] = MIGMPI_GRADIENT;
-  //  printf("%i> indeks = %li %li \n", myID, nr->indeks[0], nr->indeks[1]);
   memcpy (temp + 1, nr->param, (numelem - 1) * sizeof (double));
   memcpy (tempindex, nr->indeks, (numelem - 1) * sizeof (long));
   memcpy (temp + numelem, nr->lparam, (numelem - 1) * sizeof (double));
   temp[numelem2 - 1] = nr->profilenum;
-  for (locus = 1; locus < numcpu; locus++)
+  for (locus = 1; locus < MIN(world->loci+1,numcpu); locus++)
     {
       MPI_Send (temp, (int) numelem2, MPI_DOUBLE, locus,
-		locus, MPI_COMM_WORLD);
+		locus, comm_world);
       MPI_Send (tempindex, (int) numelem, MPI_LONG, locus,
-		locus, MPI_COMM_WORLD);
+		locus, comm_world);
     }
   memset (nr->d, 0, sizeof (double) * (numelem - 1));
-  for (locus = 1; locus < numcpu; locus++)
+  for (locus = 1; locus <  MIN(world->loci+1,numcpu); locus++)
     {
       copy_and_clear_d (nr);
       MPI_Recv (nr->d, (int) (numelem - 1), MPI_DOUBLE, MPI_ANY_SOURCE,
-		MPI_ANY_TAG, MPI_COMM_WORLD, &status);
+		MPI_ANY_TAG, comm_world, &status);
       add_back_d (nr);
       sender = status.MPI_SOURCE;
       tag = status.MPI_TAG;
@@ -326,9 +320,7 @@ mpi_gradient_worker (helper_fmt * helper, nr_fmt * nr,
     {
       locus = nr->world->who[ww];
       copy_and_clear_d (nr);
-      //      printf("called derivatives for locus %li\n",locus);
       simple_loci_derivatives (nr->d, nr, tyme, locus);
-      //      printf("%i-%li> %f %f %f %f\n", myID, locus, nr->d[0], nr->d[1], nr->d[2], nr->d[3]);
       add_back_d (nr);
     }
 }
@@ -350,24 +342,18 @@ mpi_maximize_worker (world_fmt * world, long rep)
   set_replicates (world, world->repkind, rep, &repstart, &repstop);
 
   which_calc_like (world->repkind);
-
-  MPI_Bcast (&Gmax, 1, MPI_LONG, MASTER, MPI_COMM_WORLD);
-  //  printf("%i> received broadcasted gmax=%li\n",myID,Gmax);fflush(stdout); 
+  MPI_Bcast (&Gmax, 1, MPI_LONG, MASTER, comm_world);
   create_nr (nr, world, Gmax, 0, world->loci, world->repkind, repstart);
-  setup_parameter0 (world, nr, world->repkind,
+  SETUPPARAM0 (world, nr, world->repkind,
 		    repstart, repstop, world->loci, MULTILOCUS, TRUE);
-
   while (!done)
     {
-      //      printf("%i> wait for work\n",myID);
       MPI_Recv (temp, (int) numelem2, MPI_DOUBLE, MASTER, MPI_ANY_TAG,
-		MPI_COMM_WORLD, &status);
-      //     printf("%i> got signal: %li\n",myID, (long) temp[0]);
+		comm_world, &status);
       locus = world->locus = status.MPI_TAG - 1;
       switch ((long) temp[0])
 	{
 	case MIGMPI_LIKE:
-	  //      printf("%i> in LIKE\n",myID);
 	  memset (nr->locilikes, 0, sizeof (double) * world->loci);
 	  memcpy (nr->param, temp + 1, sizeof (double) * (numelem - 1));
 	  memcpy (nr->lparam, temp + numelem,
@@ -375,22 +361,19 @@ mpi_maximize_worker (world_fmt * world, long rep)
 	  fill_helper (&helper, nr->param, nr->lparam, world, nr);
 	  mpi_likelihood_worker (world, &helper, rep);
 	  MPI_Send (nr->locilikes, (int) world->loci, MPI_DOUBLE, MASTER,
-		    locus + 1, MPI_COMM_WORLD);
+		    locus + 1, comm_world);
 	  break;
 	case MIGMPI_GRADIENT:
-	  //      printf("%i> in gradient\n",myID);
 	  memcpy (nr->param, temp + 1, sizeof (double) * (numelem - 1));
 	  memcpy (nr->lparam, temp + numelem,
 		  sizeof (double) * (numelem - 1));
 	  fill_helper (&helper, nr->param, nr->lparam, world, nr);
 	  nr->profilenum = temp[numelem2 - 1];
 	  MPI_Recv (nr->indeks, (int) numelem, MPI_LONG, MASTER, MPI_ANY_TAG,
-		    MPI_COMM_WORLD, &status);
-	  //      printf("%i> indeks = %li %li \n", myID, nr->indeks[0], nr->indeks[1]);
+		    comm_world, &status);
 	  mpi_gradient_worker (&helper, nr, world->atl);
 	  MPI_Send (nr->d, (int) nr->partsize, MPI_DOUBLE, MASTER, locus + 1,
-		    MPI_COMM_WORLD);
-	  //      printf("%i> sent derivatives\n",myID); fflush(stdout);
+		    comm_world);
 	  break;
 	case MIGMPI_RESULT:
 	  mpi_results_worker (temp[1], world, repstop, pack_result_buffer);
@@ -410,19 +393,17 @@ mpi_maximize_worker (world_fmt * world, long rep)
 	}
     }
   destroy_nr (nr, world);
-  //  printf ("%i> I am dead now\n", myID);
-  //fflush (stdout);
 }
 
-
 void
 broadcast_options_master (option_fmt * options)
 {
   int bufsize = MAXBUFSIZE;
   char *buffer;
-  buffer = (char *) calloc (MAXBUFSIZE, sizeof (char));
-  bufsize = save_options_buffer (buffer, options);
-  MPI_Bcast (buffer, bufsize, MPI_CHAR, MASTER, MPI_COMM_WORLD);
+  buffer = (char *) calloc (1, sizeof (char));
+  bufsize = save_options_buffer (&buffer, options);
+  MPI_Bcast (&bufsize, 1, MPI_LONG, MASTER, comm_world);
+  MPI_Bcast (buffer, bufsize, MPI_CHAR, MASTER, comm_world);
   free (buffer);
 }
 
@@ -430,35 +411,36 @@ broadcast_options_master (option_fmt * options)
 void
 broadcast_options_worker (option_fmt * options)
 {
-  int bufsize = MAXBUFSIZE;
+  long bufsize;
   char *buffer;
-  char *sbuffer;
-  sbuffer = (char *) calloc (MAXBUFSIZE, sizeof (char));
-  buffer = sbuffer;
-  MPI_Bcast (sbuffer, bufsize, MPI_CHAR, MASTER, MPI_COMM_WORLD);
+  MPI_Bcast (&bufsize,1, MPI_LONG, MASTER, comm_world);
+  buffer = (char *) calloc (bufsize, sizeof (char));    
+  MPI_Bcast (buffer, bufsize, MPI_CHAR, MASTER, comm_world);
   read_options_worker (&buffer, options);
-  free (sbuffer);
+  free (buffer);
 }
 
 
 void
 broadcast_data_master (data_fmt * data, option_fmt * options)
 {
-  int bufsize = MAXBUFSIZE;
+  long bufsize;
   char *buffer;
-  buffer = (char *) calloc (MAXBUFSIZE, sizeof (char));
+  buffer = (char *) calloc (1, sizeof (char));
   bufsize = pack_databuffer (&buffer, data, options);
-  MPI_Bcast (buffer, bufsize, MPI_CHAR, MASTER, MPI_COMM_WORLD);
+  MPI_Bcast (&bufsize, 1, MPI_LONG, MASTER, comm_world);
+  MPI_Bcast (buffer, bufsize, MPI_CHAR, MASTER, comm_world);
   free (buffer);
 }
 
 void
 broadcast_data_worker (data_fmt * data, option_fmt * options)
 {
-  int bufsize = MAXBUFSIZE;
+  long bufsize;
   char *buffer;
-  buffer = (char *) calloc (MAXBUFSIZE, sizeof (char));
-  MPI_Bcast (buffer, bufsize, MPI_CHAR, MASTER, MPI_COMM_WORLD);
+  MPI_Bcast (&bufsize, 1, MPI_LONG, MASTER, comm_world);
+  buffer = (char *) calloc (bufsize, sizeof (char));
+  MPI_Bcast (buffer, bufsize, MPI_CHAR, MASTER, comm_world);
   free (buffer);
 }
 
@@ -466,19 +448,25 @@ long
 pack_databuffer (char **buffer, data_fmt * data, option_fmt * options)
 {
   long locus;
-  long position = 0;
-  sprintf (*buffer, "%c%li%li%c%s\n", options->datatype, data->numpop,
+  long bufsize = 0;
+  char fp[LINESIZE];
+  bufsize += LINESIZE;
+  *buffer = (char *) realloc(*buffer, sizeof(char) * bufsize);
+  sprintf (fp, "%c%li%li%c%s\n", options->datatype, data->numpop,
 	   data->loci, data->dlm, options->title);
-  position = strlen (*buffer);
+  strcat(*buffer, fp);
   for (locus = 0; locus < data->loci; locus++)
     {
-      sprintf ((*buffer) + position, "%li ", data->seq->sites[locus]);
-      position = strlen (*buffer);
+      sprintf (fp, "%li ", data->seq->sites[locus]);
+      bufsize += strlen (fp)+1;
+     *buffer = (char *) realloc(*buffer, sizeof(char) * bufsize);
+      strcat(*buffer, fp);
     }
-  sprintf ((*buffer) + position, "\n%li %f", data->seq->addon,
-	   data->seq->fracchange);
-  position = strlen (*buffer);
-  return position;
+  sprintf (fp, "\n%li %f", data->seq->addon, data->seq->fracchange);
+        bufsize += strlen (fp)+1;
+     *buffer = (char *) realloc(*buffer, sizeof(char) * bufsize);
+      strcat(*buffer, fp);
+  return bufsize;
 }
 
 void
@@ -487,6 +475,7 @@ unpack_result_buffer (char *buffer, world_fmt * world,
 {
   long rep, pop;
   timearchive_fmt **atl = world->atl;
+  double ***apg0 = world->apg0;
   char input[LINESIZE];
   char *buf = buffer;
   strcpy (input, "");
@@ -494,12 +483,22 @@ unpack_result_buffer (char *buffer, world_fmt * world,
     {
       sgets (input, LINESIZE, &buf);
       atl[rep][locus].param_like = atof (input);
-      for (pop = 0; pop < numpop * numpop; pop++)
+      for (pop = 0; pop < 4 * numpop * numpop; pop++)
 	{
 	  sgets (input, LINESIZE, &buf);
-	  atl[rep][locus].param[pop] = atof (input);
+	  atl[rep][locus].parameters[pop] = atof (input);
 	}
     }
+// apg0
+  for (rep = 0; rep < maxrep; rep++)
+    {
+      for (pop = 0; pop < world->options->lsteps; pop++)
+	{
+	  sgets (input, LINESIZE, &buf);
+	  apg0[rep][locus][pop] = atof (input);
+	}
+    }
+
 }
 
 long
@@ -509,26 +508,36 @@ pack_result_buffer (char **buffer, world_fmt * world,
   long rep, pop;
   char input[LINESIZE];
   timearchive_fmt **atl = world->atl;
-  long temp;
-  long bufsize = strlen (*buffer);
-  strcpy (input, "");
-  if (bufsize < (temp = (maxrep * (20 + (20 * numpop * numpop)))))
-    {
-      (*buffer) = (char *) realloc (*buffer, sizeof (char) * temp);
-      bufsize = temp;
-    }
+  double ***apg0 = world->apg0;
+   long bufsize = 1;//maxrep * (1 + 4*numpop*numpop + world->options->lsteps) * 42 ;
+   (*buffer) = (char *) realloc (*buffer, sizeof (char) * bufsize);
   memset (*buffer, 0, sizeof (char) * bufsize);
   for (rep = 0; rep < maxrep; rep++)
     {
-      sprintf (input, "%f\n", atl[rep][locus].param_like);
+      sprintf (input, "%20.20f\n", atl[rep][locus].param_like);
+	bufsize += strlen(input) + 1;
+	(*buffer) = (char *) realloc (*buffer, sizeof (char) * bufsize);
       strcat (*buffer, input);
-      for (pop = 0; pop < numpop * numpop; pop++)
+      for (pop = 0; pop < 4 * numpop * numpop; pop++)
 	{
-	  sprintf (input, "%f\n", atl[rep][locus].param[pop]);
+	  sprintf (input, "%20.20f\n", atl[rep][locus].parameters[pop]);
+	bufsize += strlen(input) + 1;
+	(*buffer) = (char *) realloc (*buffer, sizeof (char) * bufsize);
 	  strcat (*buffer, input);
 	}
     }
-  return strlen (*buffer) + 1;
+// apg0
+  for (rep = 0; rep < maxrep; rep++)
+    {
+      for (pop = 0; pop < world->options->lsteps; pop++)
+	{
+	  sprintf (input, "%20.20f\n", apg0[rep][locus][pop]);
+	bufsize += strlen(input) + 1;
+   (*buffer) = (char *) realloc (*buffer, sizeof (char) * bufsize);
+	  strcat (*buffer, input);
+	}
+    }
+  return bufsize;
 }
 
 void
@@ -573,8 +582,8 @@ pack_mighist_buffer (char **buffer, world_fmt * world,
   long j, i;
   mighistloci_fmt *aa;
   char input[LINESIZE];
-  long bufsize = strlen ((*buffer));
-  aa = &world->mighistloci[locus];
+  long bufsize=1;
+    aa = &world->mighistloci[locus];
   strcpy (input, "");
   for (j = 0; j < aa->mighistnum; j++)
     bufsize += 100 + aa->mighist[j].migeventsize * 100;
@@ -615,22 +624,16 @@ unpack_sumfile_buffer (char *buffer, world_fmt * world,
       sgets (input, LINESIZE, &buf);
       sscanf (input, "%li %li %li %lf\n", &ta[r][l].T, &ta[r][l].numpop,
 	      &ta[r][l].sumtips, &ta[r][locus].param_like);
-      ta[r][l].allocT = 0;
       increase_timearchive (world, l, ta[r][l].T, world->numpop, r);
       for (i = 0; i < ta[r][l].T; i++)
 	{
 	  sgets (input, LINESIZE, &buf);
 	  sscanf (input, "%li ", &ta[r][l].tl[i].copies);
-	  for (j = 0; j < world->numpop; j++)
-	    {
+	  ta[r][l].tl[i].lcopies = ln_copies(ta[r][l].tl[i].copies);
+	  for (j = 0; j < 3*numpop + numpop * (numpop-1); j++)
+	  {
 	      sgets (input, LINESIZE, &buf);
-	      sscanf (input, "%lf %lf %lf\n", &ta[r][l].tl[i].km[j],
-		      &ta[r][l].tl[i].kt[j], &ta[r][l].tl[i].p[j]);
-	    }
-	  sgets (input, world->numpop2 * LINESIZE, &buf);
-	  for (j = 0; j < world->numpop2; j++)
-	    {
-	      sscanf (input, "%lf ", &ta[r][l].tl[i].mindex[j]);
+	      sscanf (input, "%lf\n", &ta[r][l].tl[i].data[j]);
 	    }
 	}
       for (i = 0; i < world->numpop2; i++)
@@ -656,48 +659,45 @@ pack_sumfile_buffer (char **buffer, world_fmt * world,
   char input[LINESIZE];
   timearchive_fmt **ta = world->atl;
   // calc bufsize needed
-  long bufsize = strlen ((*buffer));
-  for (r = 0; r < maxrep; r++)
-    {
-      bufsize += ta[r][l].T * (numpop * 100 + numpop2 * 20);
-      bufsize += numpop2 * 50;
-    }
+  long bufsize = 1;
   (*buffer) = (char *) realloc ((*buffer), sizeof (char) * bufsize);
   memset (*buffer, 0, sizeof (char) * bufsize);
   for (r = 0; r < maxrep; r++)
     {
-      sprintf (input, "%li %li %li %f\n", ta[r][l].T, ta[r][l].numpop,
+      sprintf (input, "%20li %20li %20li %20.20f \n", ta[r][l].T, ta[r][l].numpop,
 	       ta[r][l].sumtips, ta[r][l].param_like);
+	bufsize += strlen(input) + 1;
+   (*buffer) = (char *) realloc (*buffer, sizeof (char) * bufsize);
       strcat ((*buffer), input);
       for (i = 0; i < ta[r][l].T; i++)
 	{
-	  sprintf (input, "%li\n", ta[r][l].tl[i].copies);
+	  sprintf (input, "%20li\n", ta[r][l].tl[i].copies);
+	bufsize += strlen(input) + 1;
+   (*buffer) = (char *) realloc (*buffer, sizeof (char) * bufsize);
 	  strcat ((*buffer), input);
-	  for (j = 0; j < numpop; j++)
+	  for (j = 0; j < 3*numpop + numpop * (numpop-1); j++)
 	    {
-	      sprintf (input, "%20.20f %20.20f %f\n",
-		       ta[r][l].tl[i].km[j], ta[r][l].tl[i].kt[j],
-		       ta[r][l].tl[i].p[j]);
+	      sprintf (input, "%20.20f\n",
+		       ta[r][l].tl[i].data[j]);
+	bufsize += strlen(input) + 1;
+   (*buffer) = (char *) realloc (*buffer, sizeof (char) * bufsize);
 	      strcat ((*buffer), input);
 	    }
-	  for (j = 0; j < numpop2; j++)
-	    {
-	      sprintf (input, "%f ", ta[r][l].tl[i].mindex[j]);
-	      strcat ((*buffer), input);
-	    }
-	  sprintf (input, "\n");
-	  strcat ((*buffer), input);
 	}
       for (i = 0; i < numpop2; i++)
 	{
 	  sprintf (input, "%20.20e %20.20e\n", ta[r][l].param[i],
 		   ta[r][l].param0[i]);
+	bufsize += strlen(input) + 1;
+   (*buffer) = (char *) realloc (*buffer, sizeof (char) * bufsize);
 	  strcat ((*buffer), input);
 	}
-      sprintf (input, "%li %20.20e\n", ta[r][l].trials, ta[r][l].normd);
+      sprintf (input, "%20li %20.20e\n", ta[r][l].trials, ta[r][l].normd);
+	bufsize += strlen(input) + 1;
+   (*buffer) = (char *) realloc (*buffer, sizeof (char) * bufsize);
       strcat ((*buffer), input);
     }
-  return strlen (*buffer) + 1;
+  return bufsize;
 }
 
 void
@@ -706,9 +706,7 @@ mpi_results_master (double sendtype, world_fmt * world, long maxreplicate,
 				    long locus, long maxrep, long numpop))
 {
   long numpop = world->numpop;
-  int maxbufsize =
-    (int) (numpop * numpop * maxreplicate * sizeof (char) *
-	   MAXPRINTVALLENGTH);
+  int bufsize = 1;
   boolean done = FALSE;
   char *buffer, *sbuffer;
   double *temp;
@@ -718,13 +716,12 @@ mpi_results_master (double sendtype, world_fmt * world, long maxreplicate,
   long numelem = world->numpop2 + (world->options->gamma ? 2 : 1);
   long numelem2 = 2 * numelem;
   temp = (double *) calloc (numelem2, sizeof (double));
-
-  buffer = (char *) calloc (maxbufsize, sizeof (char));
+  buffer = (char *) calloc (bufsize, sizeof (char));
   temp[0] = sendtype;
-  temp[1] = maxbufsize;
-  for (locus = 1; locus < numcpu; locus++)
+  temp[1] = bufsize;
+  for (locus = 1; locus <  MIN(world->loci+1,numcpu); locus++)
     {
-      MPI_Send (temp, numelem2, MPI_DOUBLE, locus, locus, MPI_COMM_WORLD);
+      MPI_Send (temp, numelem2, MPI_DOUBLE, locus, locus, comm_world);
     }
   z = 0;
   while (!done)
@@ -733,23 +730,14 @@ mpi_results_master (double sendtype, world_fmt * world, long maxreplicate,
 	{
 	  break;
 	}
-      MPI_Recv (&maxbufsize, 1, MPI_LONG, MPI_ANY_SOURCE, MPI_ANY_TAG,
-		MPI_COMM_WORLD, &status);
-      printf ("%i> ----------- wait for buffer with  bufsize: %i\n", myID,
-	      maxbufsize);
-      fflush (stdout);
-
-      buffer = (char *) realloc (buffer, sizeof (char) * (maxbufsize + 1));
-      memset (buffer, 0, sizeof (char) * (maxbufsize + 1));
+      MPI_Recv (&bufsize, 1, MPI_LONG, MPI_ANY_SOURCE, MPI_ANY_TAG,
+		comm_world, &status);
+      buffer = (char *) realloc (buffer, sizeof (char) * (bufsize + 1));
+      memset (buffer, 0, sizeof (char) * (bufsize + 1));
       sender = status.MPI_SOURCE;
       tag = status.MPI_TAG;
-      MPI_Recv (buffer, maxbufsize - 1, MPI_CHAR,
-		sender, tag, MPI_COMM_WORLD, &status);
-      //printf
-      //        ("%i> have received buflen=%i from %li for locus %li and this is z=%li\n",
-      // myID, strlen (buffer), sender, tag, z);
-      //      printf ("%i> have received buffer:====\n %s\n======\n", myID, buffer);
-      //fflush (stdout);
+      MPI_Recv (buffer, bufsize, MPI_CHAR,
+		sender, tag, comm_world, &status);
       sbuffer = buffer;
       (*unpack) (buffer, world, tag - 1, maxreplicate, numpop);
     }
@@ -758,7 +746,7 @@ mpi_results_master (double sendtype, world_fmt * world, long maxreplicate,
 }
 
 void
-mpi_results_worker (long bufsize, world_fmt * world, long maxrep,
+mpi_results_worker (long bufs, world_fmt * world, long maxrep,
 		    long (*pack) (char **buffer, world_fmt * world,
 				  long locus, long maxrep, long numpop))
 {
@@ -766,25 +754,176 @@ mpi_results_worker (long bufsize, world_fmt * world, long maxrep,
   int ww, locus;
   char *buffer;
   char **buf;
-  buffer = (char *) calloc (1, sizeof (char));	//some small value
+  long bufsize = 1;
+  buffer = (char *) calloc (bufsize, sizeof (char));	//some small value
   for (ww = 0; ww < locidone; ww++)
     {
       buffer = (char *) realloc (buffer, sizeof (char));	//some small value
       buf = &buffer;
       locus = world->who[ww];
       bufsize = (*pack) (&buffer, world, locus, maxrep, numpop);
-      printf ("%i> will send locus=%i-buffer\nsssss\n %s\nssssss\n", myID,
-	      locus, buffer);
-      fflush (stdout);
-      MPI_Send (&bufsize, 1, MPI_LONG, MASTER, locus + 1, MPI_COMM_WORLD);
-      MPI_Send (buffer, bufsize - 1, MPI_CHAR, MASTER, locus + 1,
-		MPI_COMM_WORLD);
-      printf ("%i> have sent buffer with size %li\n", myID, bufsize);
-      fflush (stdout);
+	fflush (stdout);
+      MPI_Send (&bufsize, 1, MPI_LONG, MASTER, locus + 1, comm_world);
+      MPI_Send (buffer, bufsize, MPI_CHAR, MASTER, locus + 1,
+		comm_world);
     }
   free (*buf);
 }
 
+// slownet profiler
+// 
+#ifdef SLOWNET
+void mpi_broadcast_results(world_fmt *world)
+{
+   long locus;
+   long bufsize;
+   char **buffer = &world->buffer;
+   long maxreplicate = (world->options->replicate
+		       && world->options->replicatenum >
+		       0) ? world->options->replicatenum : 1;
+  MPI_Barrier(comm_world);
+	for(locus=0; locus < world->loci+1; locus++)
+  {
+ 	if(myID==MASTER)
+		bufsize = pack_result_buffer(buffer,world, locus, maxreplicate, world->numpop);
+	MPI_Bcast (&bufsize, 1, MPI_LONG, MASTER, comm_world);
+	*buffer = (char *) realloc(*buffer, sizeof(char)*bufsize+1);
+        MPI_Bcast (*buffer, bufsize, MPI_CHAR, MASTER, comm_world);
+	if(myID!=MASTER)
+		unpack_result_buffer(*buffer, world, locus, maxreplicate, world->numpop);
+	memset(*buffer,0,sizeof(char)*bufsize);	 
+  }
+  MPI_Barrier(comm_world);
+
+  for(locus=0; locus < world->loci; locus++)
+  {
+	if(myID==MASTER)
+		bufsize = pack_sumfile_buffer(buffer,world, locus, maxreplicate, world->numpop);
+	MPI_Bcast (&bufsize, 1, MPI_LONG, MASTER, comm_world);
+	*buffer = (char *) realloc(*buffer, sizeof(char)*(bufsize+1));
+        MPI_Bcast (*buffer, bufsize, MPI_CHAR, MASTER, comm_world);
+	if(myID!=MASTER)
+		unpack_sumfile_buffer(*buffer, world, locus, maxreplicate, world->numpop); 
+	memset(*buffer,0,sizeof(char)*bufsize);
+   }
+}
+
+void
+mpi_profiles_master (world_fmt *world, long nparam, int *profilewho)
+{
+  long pnum;
+  int sender;
+  int i;
+  long pdone;
+  MPI_Status status;
+  long temp[3];
+  long numsent = 0;
+  long bufsize;
+  long quantsize;
+  char **buffer = &world->buffer;
+  FILE *outfile = world->outfile;
+  for (pnum = 0; pnum < MIN (nparam, numcpu - 1); pnum++)
+    {
+      MPI_Send (&pnum, 1, MPI_LONG, pnum + 1, pnum + 1, comm_world);
+      numsent++;
+    }
+  for (pnum = 0; pnum < nparam; pnum++)
+    {   
+	   MPI_Recv (temp, 3, MPI_LONG, MPI_ANY_SOURCE, MPI_ANY_TAG,
+		comm_world, &status);
+	pdone = temp[0];
+	bufsize =temp[1];
+	quantsize = temp[2];
+      sender = status.MPI_SOURCE;
+      profilewho[pdone] = sender;
+	*buffer = (char*) realloc(*buffer, sizeof(char)*(bufsize+quantsize+1));
+       memset(*buffer,0,sizeof(char)*(bufsize+quantsize+1));
+      MPI_Recv (*buffer, bufsize+quantsize, MPI_CHAR, sender, MPI_ANY_TAG, comm_world, &status);
+      unpack_quantile((*buffer)+bufsize, world->quantiles[pdone], GRIDSIZE);
+	memset((*buffer)+bufsize,0,sizeof(char)*quantsize);	 
+      fprintf(outfile,"%s\n\n",*buffer);
+      if (numsent < nparam)
+	{
+	  MPI_Send (&numsent, 1, MPI_LONG, sender, numsent + 1,
+		    comm_world);
+	  numsent++;
+	}
+      else
+	{
+	  MPI_Send (&nparam, 1, MPI_LONG, sender, 0, comm_world);	//end of parameter list
+	}
+    }
+}
+
+void
+mpi_profiles_worker (world_fmt *world, long *gmaxptr)
+{
+  boolean done = FALSE;
+  long pnum;
+  long temp[3];
+  char *quantilebuffer;
+  MPI_Status status;
+  long qbufsize = 1;
+  quantilebuffer = (char *) calloc(qbufsize, sizeof(char));
+  while (!done)
+    {
+      MPI_Recv (&pnum, 1, MPI_LONG, MASTER, MPI_ANY_TAG,
+		comm_world, &status);
+      if (status.MPI_TAG != 0)	//stop condition
+	{
+	  print_profile_likelihood_driver (pnum, world, gmaxptr);
+	  temp[0] = pnum;
+	  temp[1] = strlen(world->buffer);
+	   temp[2] = pack_quantile(&quantilebuffer, world->quantiles[pnum], GRIDSIZE);
+	  world->buffer = (char*) realloc(world->buffer, sizeof(char) * (temp[1] + temp[2] + 1));
+	  strcat(world->buffer,quantilebuffer);
+	  MPI_Send (temp, 3, MPI_LONG, MASTER, pnum + 1, comm_world);
+	  MPI_Send (world->buffer, temp[1]+temp[2], MPI_CHAR, MASTER, pnum + 1, comm_world);
+	  world->profilewho[profiledone++] = pnum;
+	}
+      else
+	{
+	  done = TRUE;
+	}
+    }
+	free(quantilebuffer);
+}
+
+long pack_quantile(char **buffer, quantile_fmt quant, long n)
+{
+	long i;
+	char fp[LINESIZE];
+	long bufsize=LINESIZE;
+	*buffer = (char *) realloc(*buffer, sizeof(char) * bufsize);
+	sprintf(*buffer,"QUANTILEBUFFER:\n %s\n",quant.name);
+	for(i=0; i<n; i++)
+	{
+		sprintf(fp,"%20.20f\n",quant.param[i]);
+		bufsize += strlen(fp) +1;
+		*buffer = (char *) realloc(*buffer, sizeof(char) * bufsize);
+		strcat(*buffer,fp);
+	}
+	return bufsize;
+}
+
+void unpack_quantile(char *buffer, quantile_fmt quant, long n)
+{
+	long i;
+	char input[LINESIZE];
+	char *buf = buffer;
+	strcpy (input, "");
+	 sgets (input, LINESIZE, &buf);
+	 sgets (input, LINESIZE, &buf);
+	strcpy(quant.name,input);
+	for(i=0; i<n; i++)
+	{
+	     sgets (input, LINESIZE, &buf);
+	     quant.param[i] = atof(input);
+	}
+}
+
+#endif
+
 /*
 // send the data over all loci/replicates to all nodes
 // including the master node, so that all nodes can then 
@@ -818,11 +957,12 @@ assignloci_worker (world_fmt * world)
   while (!done)
     {
       MPI_Recv (&locus, 1, MPI_LONG, MASTER, MPI_ANY_TAG,
-		MPI_COMM_WORLD, &status);
+		comm_world, &status);
       if (status.MPI_TAG != 0)	//stop condition
 	{
 	  swap_atl (locus, locidone, world);
-	  MPI_Send (&locus, 1, MPI_LONG, MASTER, locus + 1, MPI_COMM_WORLD);
+
+	  MPI_Send (&locus, 1, MPI_LONG, MASTER, locus + 1, comm_world);
 	  /* we want to know what locus we worked for
 	     - to control the work sent by master
 	     - to use in setup_parameter0() [combroyden2.c] */
@@ -848,4 +988,125 @@ swap_atl (long from, long to, world_fmt * world)
     }
 }
 
+
+#ifdef SLOWNET
+void setup_parameter0_slowmpi(world_fmt * world, nr_fmt * nr, long repkind, long repstart,
+		  long repstop, long loci, long kind, boolean multilocus)
+{
+  long locus, r;
+  if (myID != MASTER)
+    {
+      if (multilocus)
+	{
+	for (locus = 0; locus < loci; locus++)
+	    {
+		 if (repkind == SINGLECHAIN)
+		{
+		  for (r = repstart; r < repstop; r++)
+		    create_apg0 (nr->apg0[r][locus], nr,
+				 &world->atl[r][locus]);
+		}
+	      else
+		{
+		  if (kind != PROFILE)
+		    {
+		      for (r = repstart; r < repstop; r++)
+			create_apg0 (nr->apg0[r][locus], nr,
+				     &world->atl[r][locus]);
+		      interpolate_like (nr, locus);
+		    }
+		  else
+		    {
+		      for (r = repstart; r < repstop; r++)
+			create_multiapg0 (nr->apg0[r][locus], nr, r, locus);
+		    }
+		}
+	    }
+	}
+      else			//single locus
+	{
+	  if (repkind == SINGLECHAIN)
+	    {
+	      for (r = repstart; r < repstop; r++)
+		create_apg0 (nr->apg0[r][world->locus], nr,
+			     &world->atl[r][world->locus]);
+	    }
+	  else
+	    {
+	      if (kind != PROFILE)
+		{
+		  for (r = repstart; r < repstop; r++)
+		    create_apg0 (nr->apg0[r][world->locus], nr,
+				 &world->atl[r][world->locus]);
+		  interpolate_like (nr, world->locus);
+		}
+	      for (r = repstart; r < repstop; r++)
+		create_multiapg0 (nr->apg0[r][world->locus], nr, r,
+				  world->locus);
+	    }
+	}
+    }
+}
+#endif
+
+void setup_parameter0_mpi(world_fmt * world, nr_fmt * nr, long repkind, long repstart,
+		  long repstop, long loci, long kind, boolean multilocus)
+{
+  long locus, r;
+  long ll;
+  if (myID != MASTER)
+    {
+      if (multilocus)
+	{
+	  for (ll = 0; ll < locidone; ll++)
+	    {
+	      locus = world->locus = world->who[ll];
+	      if (repkind == SINGLECHAIN)
+		{
+		  for (r = repstart; r < repstop; r++)
+		    create_apg0 (nr->apg0[r][locus], nr,
+				 &world->atl[r][locus]);
+		}
+	      else
+		{
+		  if (kind != PROFILE)
+		    {
+		      for (r = repstart; r < repstop; r++)
+			create_apg0 (nr->apg0[r][locus], nr,
+				     &world->atl[r][locus]);
+		      interpolate_like (nr, locus);
+		    }
+		  else
+		    {
+		      for (r = repstart; r < repstop; r++)
+			create_multiapg0 (nr->apg0[r][locus], nr, r, locus);
+		    }
+		}
+	    }
+	}
+      else			//single locus
+	{
+	  if (repkind == SINGLECHAIN)
+	    {
+	      for (r = repstart; r < repstop; r++)
+		create_apg0 (nr->apg0[r][world->locus], nr,
+			     &world->atl[r][world->locus]);
+	    }
+	  else
+	    {
+	      if (kind != PROFILE)
+		{
+		  for (r = repstart; r < repstop; r++)
+		    create_apg0 (nr->apg0[r][world->locus], nr,
+				 &world->atl[r][world->locus]);
+		  interpolate_like (nr, world->locus);
+		}
+	      for (r = repstart; r < repstop; r++)
+		create_multiapg0 (nr->apg0[r][world->locus], nr, r,
+				  world->locus);
+	    }
+	}
+    }
+}
+
 #endif
