@@ -1,9 +1,18 @@
 #ifndef __MIGRATE_MPI_
 #define __MIGRATE_MPI_
-
+#ifdef MPI
+#include <mpi.h>
+#include "migration.h"
+extern MPI_Comm comm_world;
+extern MPI_Comm comm_all_worlds;
+#endif
 extern int myID;
+extern int allID;
+extern int color;
 extern int numcpu;
+extern int numlocicpu;
 extern int locidone;
+extern int profiledone;
 
 #define MASTER 0
 #define SLAVE 1
@@ -17,10 +26,6 @@ extern int locidone;
 
 #ifdef MPI
 
-
-#include <mpi.h>
-#include "migration.h"
-
 #ifndef PTHREADS
 #define tpool_t char
 #else
@@ -78,5 +83,14 @@ extern void mpi_gmax_worker (world_fmt * world);
 extern void mpi_send_stop (world_fmt * world);
 extern void mpi_results_stop (void);
 extern void assignloci_worker (world_fmt * world);
-#endif
+void setup_parameter0_mpi(world_fmt * world, nr_fmt * nr, long repkind, long repstart,
+		  long repstop, long loci, long kind, boolean multilocus);
+#ifdef SLOWNET
+void mpi_broadcast_results(world_fmt *world);
+void mpi_profiles_master (world_fmt *world, long nparam, int *profilewho);
+void mpi_profiles_worker (world_fmt *world, long *gmaxptr);
+void setup_parameter0_slowmpi(world_fmt * world, nr_fmt * nr, long repkind, long repstart,
+		  long repstop, long loci, long kind, boolean multilocus);
+#endif /*slownet*/
+#endif /*mpi*/
 #endif
