@@ -25,10 +25,10 @@
 
 typedef char allele_type[DEFAULT_ALLELENMLENGTH];
 
+typedef double pair[2]; 
 typedef double sitelike[4];
 typedef sitelike *ratelike;
 typedef ratelike *phenotype;
-
 typedef double contribarr[MAXCATEGS];
 typedef short val[MAXCATEGS];
 
@@ -53,7 +53,8 @@ lratio_fmt;
 
 typedef struct valrec
 {
-  double rat, ratxi, ratxv, zz, z1, y1, ww1, zz1, ww2, zz2, z1zz, z1yy, xiz1, xiy1xv, ww1zz1, vv1zz1, ww2zz2, vv2zz2;
+  double rat, ratxi, ratxv, zz, z1, y1, ww1, zz1, ww2, zz2, z1zz, z1yy, xiz1,
+    xiy1xv, ww1zz1, vv1zz1, ww2zz2, vv2zz2;
 }
 valrec;
 
@@ -66,6 +67,15 @@ typedef union xarray_fmt
 }
 xarray_fmt;
 
+#ifdef UEP
+typedef union ueparray_fmt
+{
+  double *a;
+  pair * s;
+}
+ueparray_fmt;
+#endif
+
 typedef struct seqmodel_fmt
 {
   double freqa, freqt, freqg, freqc;
@@ -102,6 +112,8 @@ typedef struct _data
   FILE *uepfile;
   long **uep;
   long uepsites;
+  double uepfreq0;
+  double uepfreq1;
 #endif
   char *****yy;
   double *geo;
@@ -129,7 +141,7 @@ typedef long quad_fmt[4];
 
 typedef struct _option
 {
-  char **buffer; //pointer to buffer when MPI-worker otherwise NULL
+  char **buffer;		//pointer to buffer when MPI-worker otherwise NULL
   FILE *parmfile;
   FILE *seedfile;
   FILE *logfile;
@@ -137,8 +149,8 @@ typedef struct _option
   long nmlength;		/* length of individual names */
   long popnmlength;		/* length of population names */
   long allelenmlength;		/* length of allele names */
-  char *custm; /* custom migration matrix: theta are first */
-  char *custm2; /*custom migration matrix: theta on diagonal*/
+  char *custm;			/* custom migration matrix: theta are first */
+  char *custm2;			/*custom migration matrix: theta on diagonal */
   long zeron;
   long *zeroparam;
   long constn;
@@ -157,12 +169,16 @@ typedef struct _option
   boolean writelog;
   boolean uep;
   double ueprate;
+  double uepmu;
+  double uepnu;
   boolean movingsteps;
   double acceptfreq;
   boolean printdata;
   boolean usertree;
   boolean randomtree;
   short treeprint;
+  boolean usem;
+  short migvar;
   boolean plot;
   boolean plotnow;
   short plotmethod;
@@ -198,6 +214,8 @@ typedef struct _option
   boolean aic;
   boolean fast_aic;
   double aicmod;
+  FILE *aicfile;
+  char aicfilename[256];
   char fsttype;
   boolean printfst;
   short profile;
@@ -258,7 +276,7 @@ typedef struct _option
   long lsteps;
   double lcepsilon;
   boolean gelman;
-  long pluschain;//how many chains are allowed , currently HARDCODED
+  long pluschain;		//how many chains are allowed , currently HARDCODED
   boolean replicate;
   long replicatenum;
   long gridpoints;
@@ -288,6 +306,8 @@ typedef struct _node
   xarray_fmt x;
 #ifdef UEP
   long *uep;
+  ueparray_fmt ux;
+  double uepscale;
 #endif
   double *s;
   double *scale;
@@ -339,12 +359,13 @@ typedef struct tarchive_fmt
 {
   long copies;
   double lcopies;
-  double *point;
-  double *wait;
-  double *km;
-  double *kt;
-  double *p;
-  double *mindex;
+  double *data; //hold content
+  double *point;  // points into data
+  double *wait; // points into data
+  double *kt;  //points to wait
+  double *km; //points to wait
+  double *p;   // points to point
+  double *mindex; // points to point
 }
 tarchive_fmt;
 
@@ -358,11 +379,11 @@ typedef struct timearchive_fmt
   double thb;
   double alpha;
   tarchive_fmt *tl;
-  double *parameters;
-  double *param;
-  double *param0;
-  double *lparam0;
-  double *likelihood;
+  double *parameters;// holds data
+  double *param; // pointer into parameters
+  double *param0; //pointer into parameters
+  double *lparam0; //pointer into parameters
+  double *likelihood; //pointer into parameters
   long trials;
   double normd;
 }
@@ -390,10 +411,10 @@ typedef double migevent_fmt[3];
 
 typedef struct _mighist_fmt
 {
-  migevent_fmt *migevents;
-  long migeventsize;
   long copies;
-  double weight;
+  long weight;
+  long migeventsize;
+  migevent_fmt *migevents;
 }
 mighist_fmt;
 
@@ -404,6 +425,14 @@ typedef struct _mighistloci_fmt
 }
 mighistloci_fmt;
 
+typedef struct _histogram_fmt
+{
+  long count;
+  double *time;
+  long *weight;
+}
+histogram_fmt;
+
 typedef struct _worldoption
 {
   boolean gamma;
@@ -460,6 +489,8 @@ typedef struct _worldoption
   long pluschain;
   boolean mighist;
   long burn_in;
+  boolean usem;
+  short migvar;
   boolean plot;
   long plotmethod;
   long plotintervals;
@@ -472,10 +503,13 @@ typedef struct _worldoption
   boolean aic;
   boolean fast_aic;
   double aicmod;
+  FILE *aicfile;
   double lambda;
 #ifdef UEP
   boolean uep;
   double ueprate;
+  double uepmu;
+  double uepnu;
 #endif
   boolean fastlike;
 }
@@ -484,10 +518,11 @@ worldoption_fmt;
 typedef struct _ueptime
 {
   long size;
-  long   *populations; // population the nodes are in
-  double *ueptime;    // first elementis bottom most time , last elem is
-                      // topmost time,in between are for each migration events
-} ueptime_fmt;
+  long *populations;		// population the nodes are in
+  double *ueptime;		// first elementis bottom most time , last elem is
+  // topmost time,in between are for each migration events
+}
+ueptime_fmt;
 
 typedef struct _worlddata
 {
@@ -501,6 +536,8 @@ typedef struct _worlddata
   double freqlast;
 #ifdef UEP
   long uepsites;
+  double uepfreq0;
+  double uepfreq1;
 #endif
 }
 worlddata_fmt;
@@ -514,7 +551,11 @@ typedef struct _world
   worlddata_fmt *data;
 #ifdef MPI
   int *who;
-#endif 
+#ifdef SLOWNET
+  int *profilewho;
+#endif
+#endif
+  char *buffer;	 // buffer for profiles, not needed before profiletables()
   long loci;
   long skipped;			/*loci with no data */
   long locus;			/* the current locus, if single then set to 0 */
@@ -553,6 +594,9 @@ typedef struct _world
   double **migproblist;
   long *design0list;
 
+	/*nr related */
+	  double ***apg0;		/* part-loglikelihoods of param0 */
+  double ***apg;		/* part-loglikelihoods of param */
 
   /*heating scheme */
   long actualinc;
@@ -595,7 +639,7 @@ typedef struct _world
   double ****ueplikestore;	// constains all ueplike long-steps x pop x uep
   ueptime_fmt *ueptime;		//contains ueptime per tree and mutation
   ueptime_fmt ***ueptimestore;	//contains all ueptimes
-  long ***ueprootstore;        //stores uep status at root
+  long ***ueprootstore;		//stores uep status at root
   long *oldrootuep;
   long **uepanc;
 #endif
@@ -618,7 +662,7 @@ typedef struct _nr_fmt
   long partsize;		/*number of part-variables, fixed per model */
   double *parts;		/* parts of the first and second derivatives */
   double *d;			/* first derivates */
-  double *saved;		/* saved first derivates (used in MPI code)*/
+  double *saved;		/* saved first derivates (used in MPI code) */
   /* migration parameter array  starts and ends */
   int *mstart;			//link to world->mstart
   int *mend;			//link to world->mend
@@ -635,9 +679,9 @@ typedef struct _nr_fmt
   world_fmt *world;
   double **dd;			/* second derivatives */
   double *param;		/* changed values of param */
-  //double *oparam;		/* saved old values of param */
+  //double *oparam;             /* saved old values of param */
   double *lparam;		/* changed values of log(param) */
-  //  double *olparam;		/* saved old log(parameters) */
+  //  double *olparam;          /* saved old log(parameters) */
   double llike;			/* parameter LOGlikelihood */
   double lastllike;		/* parameter LOGlikelihood */
   double ollike;		/* old "   " */
@@ -646,9 +690,9 @@ typedef struct _nr_fmt
   double ***apg0;		/* part-loglikelihoods of param0 */
   double ***apg;		/* part-loglikelihoods of param */
   double *apg_max;		/* maxvalue of apg */
-  double *rate;                 /* rates for gamma rates*/
-  double *probcat;              /* probabilities for gamma rates*/
-  long categs;                  /* #categories for gamma rates*/
+  double *rate;			/* rates for gamma rates */
+  double *probcat;		/* probabilities for gamma rates */
+  long categs;			/* #categories for gamma rates */
   double alpha;
   long numpop;			/* number of populations */
   long numpop2;			/* 2*numpop */
@@ -659,7 +703,7 @@ typedef struct _nr_fmt
   long profilenum;		/* number of profile parameters */
   long *profiles;		/* which profile parameters */
   long *indeks;			/* which noprofile parameters */
-  double *values;               /* profile values*/
+  double *values;		/* profile values */
 }
 nr_fmt;
 
@@ -723,7 +767,11 @@ typedef struct proposal_fmt
   double likelihood;
   double ueplikelihood;
   double **ueplike;
+#ifdef UEP
+  ueparray_fmt ut;
+  ueparray_fmt uf;
   node *firstuep;
+#endif
   double time;
   double v;
   double vs;
@@ -731,6 +779,10 @@ typedef struct proposal_fmt
   xarray_fmt xf;
   double *mf;
   double *mt;
+#ifdef UEP
+  double *umf;
+  double *umt;
+#endif
   node **aboveorigin;
   node **bordernodes;
   long listsize;
@@ -773,7 +825,8 @@ locusdata_fmt;
 
 
 
-typedef struct _aic_fmt {
+typedef struct _aic_fmt
+{
   double aic;
   char *pattern;
   long numparam;
@@ -781,8 +834,24 @@ typedef struct _aic_fmt {
   double lrt;
   double prob;
   double probcorr;
-} aic_fmt;
+}
+aic_fmt;
 
+typedef struct _plotfield_fmt
+{
+  boolean print;
+  char type;			// 'a' = ascii plot, 'p' = postscript plot
+  long xsize;			// width  printpositions in chars
+  long ysize;			// height printpositions in chars
+  char xaxis[255];		//xaxis label
+  char yaxis[255];		//yaxis label
+  char yfaxis[255];		//frequency yaxis label
+  char title[255];
+  double *yfreq;
+  long *y;
+  char **data;			//the plotplane
+}
+plotfield_fmt;
 
 /* here a some definitions which should be remove in a future version
    (a) FLOC : place holder for first locus strcuture should finally deal
@@ -796,6 +865,12 @@ extern char appl[20];
 typedef long longer[3];		/* used in random.c */
 extern long *seed;
 extern long *newseed;
+#ifdef SLOWNET
+extern  double (*calc_like) (helper_fmt *, double *, double *);
+extern void (*calc_gradient) (nr_fmt *, helper_fmt *, double *);
+extern  void (*setup_param0) (world_fmt * , nr_fmt * , long , long , long , long , long , boolean );
+
+#endif
 
 #include "tools.h"
 #include "sort.h"
