@@ -14,7 +14,7 @@
  
 Copyright 2001 Peter Beerli and Joseph Felsenstein
 
-$Id: tools.c,v 1.48 2001/07/25 19:27:24 beerli Exp $
+$Id: tools.c,v 1.49 2001/09/07 23:56:13 beerli Exp $
 -------------------------------------------------------*/
 #include "migration.h"
 #include "data.h"
@@ -77,6 +77,7 @@ void write_savesum (world_fmt * world);
 void translate (char *text, char from, char to);
 long count_words (char *text);
 void print_line (FILE * outfile, char c, long nn, long flag);
+void sprint_line (char *buffer, char c, long nn, long flag);
 
 /* time reporting */
 void get_time (char *nowstr, char ts[]);
@@ -381,7 +382,6 @@ openfile (FILE ** fp, char *filename, char *mode, const char *application,
   FILE *of = NULL;
   char file[LINESIZE];
   char *p;
-
   if ((p = strpbrk (filename, CRLF)) != NULL)
     *p = '\0';
   strncpy (file, filename, LINESIZE - 1);
@@ -2239,6 +2239,38 @@ print_line (FILE * outfile, char c, long nn, long flag)
   fputc ('\n', outfile);
 }
 
+
+void
+sprint_line (char * buffer, char c, long nn, long flag)
+{
+char ch[2];
+  long i, start = 0;
+  char fp[LINESIZE];
+  buffer[0]='\0';
+  ch[0]=c;
+  ch[1]='\0';
+  switch (flag)
+    {
+    case START:
+      start = 2;
+      sprintf(fp,"=%c%c",c,c);
+       strcat(buffer,fp);
+      break;
+    case STOP:
+      start = 2;
+      sprintf(fp,"==%c",c);
+       strcat(buffer,fp);
+  break;
+    default:
+      start = 0;
+    }
+  for (i = start; i < nn; i++)
+    {
+      strcat (buffer, ch);
+    }
+  strcat (buffer,"\n");
+}
+
 char
 sgetc (char **buffer)
 {
